@@ -20,6 +20,7 @@ package index
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"io"
 
@@ -28,7 +29,6 @@ import (
 	"github.com/apache/skywalking-banyandb/api/common"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
-	"github.com/apache/skywalking-banyandb/banyand/observability"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting"
 )
@@ -49,14 +49,22 @@ func (f FieldKey) MarshalIndexRule() string {
 
 // Marshal encodes f to bytes.
 func (f FieldKey) Marshal() []byte {
-	s := f.SeriesID.Marshal()
+	var s []byte
+	if f.HasSeriesID() {
+		s = f.SeriesID.Marshal()
+	}
 	i := []byte(f.MarshalIndexRule())
 	b := make([]byte, len(s)+len(i))
-	bp := copy(b, s)
-	copy(b[bp:], i)
+	copy(b, s)
+	copy(b[len(s):], i)
 	return b
 }
 
+// HasSeriesID reports whether f has a series id.
+func (f FieldKey) HasSeriesID() bool {
+	return f.SeriesID > 0
+}
+
 // MarshalToStr encodes f to string.
 func (f FieldKey) MarshalToStr() string {
 	return string(f.Marshal())
@@ -157,7 +165,7 @@ func (r RangeOpts) Between(value []byte) int {
 // FieldIterator allows iterating over a field's posting values.
 type FieldIterator interface {
 	Next() bool
-	Val() *PostingValue
+	Val() (uint64, common.SeriesID)
 	Close() error
 }
 
@@ -170,28 +178,40 @@ func (i *dummyIterator) Next() bool {
 	return false
 }
 
-func (i *dummyIterator) Val() *PostingValue {
-	return nil
+func (i *dummyIterator) Val() (uint64, common.SeriesID) {
+	return 0, 0
 }
 
 func (i *dummyIterator) Close() error {
 	return nil
 }
 
-// PostingValue is the collection of a field's values.
-type PostingValue struct {
-	Value posting.List
-	Term  []byte
+// Document represents a document in a index.
+type Document struct {
+	Fields       []Field
+	EntityValues []byte
+	DocID        uint64
 }
 
-// Writer allows writing fields and itemID in a document to a index.
+// Documents is a collection of documents.
+type Documents []Document
+
+// Batch is a collection of documents.
+type Batch struct {
+	Applied   chan struct{}
+	Documents Documents
+}
+
+// Writer allows writing fields and docID in a document to a index.
 type Writer interface {
-	Write(fields []Field, itemID common.ItemID) error
+	Write(fields []Field, docID uint64) error
+	Batch(batch Batch) error
 }
 
 // FieldIterable allows building a FieldIterator.
 type FieldIterable interface {
-	Iterator(fieldKey FieldKey, termRange RangeOpts, order modelv1.Sort) (iter FieldIterator, err error)
+	Iterator(fieldKey FieldKey, termRange RangeOpts, order modelv1.Sort, preLoadSize int) (iter FieldIterator, err error)
+	Sort(sids []common.SeriesID, fieldKey FieldKey, order modelv1.Sort, preLoadSize int) (FieldIterator, error)
 }
 
 // Searcher allows searching a field either by its key or by its key and term.
@@ -205,10 +225,41 @@ type Searcher interface {
 
 // Store is an abstract of a index repository.
 type Store interface {
-	observability.Observable
 	io.Closer
 	Writer
 	Searcher
+	SizeOnDisk() int64
+}
+
+// Series represents a series in a index.
+type Series struct {
+	EntityValues []byte
+	ID           common.SeriesID
+}
+
+// SeriesStore is an abstract of a series repository.
+type SeriesStore interface {
+	Store
+	// Search returns a list of series that match the given matchers.
+	Search(context.Context, []SeriesMatcher) ([]Series, error)
+}
+
+// SeriesMatcherType represents the type of series matcher.
+type SeriesMatcherType int
+
+const (
+	// SeriesMatcherTypeExact represents an exact matcher.
+	SeriesMatcherTypeExact SeriesMatcherType = iota
+	// SeriesMatcherTypePrefix represents a prefix matcher.
+	SeriesMatcherTypePrefix
+	// SeriesMatcherTypeWildcard represents a wildcard matcher.
+	SeriesMatcherTypeWildcard
+)
+
+// SeriesMatcher represents a series matcher.
+type SeriesMatcher struct {
+	Match []byte
+	Type  SeriesMatcherType
 }
 
 // GetSearcher returns a searcher associated with input index rule type.
