@@ -21,8 +21,11 @@ package inverted
 import (
 	"bytes"
 	"context"
+	"errors"
+	"io"
 	"log"
 	"math"
+	"sync/atomic"
 	"time"
 
 	"github.com/blugelabs/bluge"
@@ -30,13 +33,11 @@ import (
 	"github.com/blugelabs/bluge/analysis/analyzer"
 	blugeIndex "github.com/blugelabs/bluge/index"
 	"github.com/blugelabs/bluge/search"
-	"github.com/dgraph-io/badger/v3/y"
 	"go.uber.org/multierr"
 
 	"github.com/apache/skywalking-banyandb/api/common"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
-	"github.com/apache/skywalking-banyandb/banyand/observability"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting"
@@ -46,15 +47,17 @@ import (
 )
 
 const (
-	docID         = "_id"
+	docIDField    = "_id"
 	batchSize     = 1024
 	seriesIDField = "series_id"
+	entityField   = "entity"
 	idField       = "id"
 )
 
 var (
-	defaultUpper = convert.Uint64ToBytes(math.MaxUint64)
-	defaultLower = convert.Uint64ToBytes(0)
+	defaultUpper            = convert.Uint64ToBytes(math.MaxUint64)
+	defaultLower            = convert.Uint64ToBytes(0)
+	defaultRangePreloadSize = 1000
 )
 
 var analyzers map[databasev1.IndexRule_Analyzer]*analysis.Analyzer
@@ -76,11 +79,6 @@ type StoreOpts struct {
 	BatchWaitSec int64
 }
 
-type doc struct {
-	fields []index.Field
-	itemID common.ItemID
-}
-
 type flushEvent struct {
 	onComplete chan struct{}
 }
@@ -90,16 +88,29 @@ type store struct {
 	ch            chan any
 	closer        *run.Closer
 	l             *logger.Logger
+	errClosing    atomic.Pointer[error]
 	batchInterval time.Duration
 }
 
+func (s *store) Batch(batch index.Batch) error {
+	if !s.closer.AddRunning() {
+		return nil
+	}
+	defer s.closer.Done()
+	select {
+	case <-s.closer.CloseNotify():
+	case s.ch <- batch:
+	}
+	return nil
+}
+
 // NewStore create a new inverted index repository.
-func NewStore(opts StoreOpts) (index.Store, error) {
-	indexConfig := blugeIndex.DefaultConfig(opts.Path).WithUnsafeBatches().
-		WithPersisterNapTimeMSec(60 * 1000)
-	indexConfig.MergePlanOptions.MaxSegmentsPerTier = 1
-	indexConfig.MergePlanOptions.MaxSegmentSize = 500000
-	indexConfig.MergePlanOptions.SegmentsPerMergeTask = 20
+func NewStore(opts StoreOpts) (index.SeriesStore, error) {
+	indexConfig := blugeIndex.DefaultConfig(opts.Path)
+	if opts.BatchWaitSec > 0 {
+		indexConfig = indexConfig.WithUnsafeBatches().
+			WithPersisterNapTimeMSec(int(opts.BatchWaitSec * 1000))
+	}
 	config := bluge.DefaultConfigWithIndexConfig(indexConfig)
 	config.DefaultSearchAnalyzer = analyzers[databasev1.IndexRule_ANALYZER_KEYWORD]
 	config.Logger = log.New(opts.Logger, opts.Logger.Module(), 0)
@@ -122,16 +133,16 @@ func NewStore(opts StoreOpts) (index.Store, error) {
 	return s, nil
 }
 
-func (s *store) Stats() observability.Statistics {
-	return observability.Statistics{}
-}
-
 func (s *store) Close() error {
+	s.flush()
 	s.closer.CloseThenWait()
-	return s.writer.Close()
+	if s.errClosing.Load() != nil {
+		return *s.errClosing.Load()
+	}
+	return nil
 }
 
-func (s *store) Write(fields []index.Field, itemID common.ItemID) error {
+func (s *store) Write(fields []index.Field, docID uint64) error {
 	if len(fields) < 1 {
 		return nil
 	}
@@ -141,64 +152,67 @@ func (s *store) Write(fields []index.Field, itemID common.ItemID) error {
 	defer s.closer.Done()
 	select {
 	case <-s.closer.CloseNotify():
-	case s.ch <- doc{
-		fields: fields,
-		itemID: itemID,
+	case s.ch <- index.Document{
+		Fields: fields,
+		DocID:  docID,
 	}:
 	}
 	return nil
 }
 
-func (s *store) Iterator(fieldKey index.FieldKey, termRange index.RangeOpts, order modelv1.Sort) (iter index.FieldIterator, err error) {
+func (s *store) Iterator(fieldKey index.FieldKey, termRange index.RangeOpts, order modelv1.Sort, preLoadSize int) (iter index.FieldIterator, err error) {
 	if termRange.Lower != nil &&
 		termRange.Upper != nil &&
 		bytes.Compare(termRange.Lower, termRange.Upper) > 0 {
 		return index.DummyFieldIterator, nil
 	}
-	if termRange.Upper == nil {
-		termRange.Upper = defaultUpper
-	}
-	if termRange.Lower == nil {
-		termRange.Lower = defaultLower
-	}
+
 	reader, err := s.writer.Reader()
 	if err != nil {
 		return nil, err
 	}
 	fk := fieldKey.MarshalIndexRule()
-	var query bluge.Query
-	shouldDecodeTerm := true
-	if fieldKey.Analyzer == databasev1.IndexRule_ANALYZER_UNSPECIFIED {
-		query = bluge.NewTermRangeInclusiveQuery(
-			index.FieldStr(fieldKey, termRange.Lower),
-			index.FieldStr(fieldKey, termRange.Upper),
+	query := bluge.NewBooleanQuery()
+	addRange := func(query *bluge.BooleanQuery, termRange index.RangeOpts) *bluge.BooleanQuery {
+		if termRange.Upper == nil {
+			termRange.Upper = defaultUpper
+		}
+		if termRange.Lower == nil {
+			termRange.Lower = defaultLower
+		}
+		bluge.NewMatchAllQuery()
+		query.AddMust(bluge.NewTermRangeInclusiveQuery(
+			string(termRange.Lower),
+			string(termRange.Upper),
 			termRange.IncludesLower,
 			termRange.IncludesUpper,
 		).
-			SetField(fk)
+			SetField(fk))
+		return query
+	}
+
+	if fieldKey.HasSeriesID() {
+		query = query.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).
+			SetField(seriesIDField))
+		if termRange.Lower != nil || termRange.Upper != nil {
+			query = addRange(query, termRange)
+		}
 	} else {
-		shouldDecodeTerm = false
-		query = bluge.NewBooleanQuery().
-			AddMust(bluge.NewTermRangeInclusiveQuery(
-				string(termRange.Lower),
-				string(termRange.Upper),
-				termRange.IncludesLower,
-				termRange.IncludesUpper,
-			).
-				SetField(fk)).
-			AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).SetField(seriesIDField))
+		query = addRange(query, termRange)
 	}
 
 	sortedKey := fk
 	if order == modelv1.Sort_SORT_DESC {
 		sortedKey = "-" + sortedKey
 	}
-	documentMatchIterator, err := reader.Search(context.Background(), bluge.NewTopNSearch(math.MaxInt64, query).SortBy([]string{sortedKey}))
-	if err != nil {
-		return nil, err
+	result := &sortIterator{
+		query:     query,
+		reader:    reader,
+		sortedKey: sortedKey,
+		size:      preLoadSize,
+		sid:       fieldKey.SeriesID,
 	}
-	result := newBlugeMatchIterator(documentMatchIterator, fk, shouldDecodeTerm)
-	return &result, nil
+	return result, nil
 }
 
 func (s *store) MatchField(fieldKey index.FieldKey) (list posting.List, err error) {
@@ -211,26 +225,25 @@ func (s *store) MatchTerms(field index.Field) (list posting.List, err error) {
 		return nil, err
 	}
 	fk := field.Key.MarshalIndexRule()
-	var query bluge.Query
-	shouldDecodeTerm := true
-	if field.Key.Analyzer == databasev1.IndexRule_ANALYZER_UNSPECIFIED {
-		query = bluge.NewTermQuery(string(field.Marshal())).SetField(fk)
-	} else {
-		shouldDecodeTerm = false
-		query = bluge.NewBooleanQuery().
-			AddMust(bluge.NewTermQuery(string(field.Term)).SetField(fk)).
-			AddMust(bluge.NewTermQuery(string(field.Key.SeriesID.Marshal())).SetField(seriesIDField))
+	query := bluge.NewBooleanQuery().
+		AddMust(bluge.NewTermQuery(string(field.Term)).SetField(fk))
+	if field.Key.HasSeriesID() {
+		query = query.AddMust(bluge.NewTermQuery(string(field.Key.SeriesID.Marshal())).
+			SetField(seriesIDField))
 	}
 	documentMatchIterator, err := reader.Search(context.Background(), bluge.NewAllMatches(query))
 	if err != nil {
 		return nil, err
 	}
-	iter := newBlugeMatchIterator(documentMatchIterator, fk, shouldDecodeTerm)
+	iter := newBlugeMatchIterator(documentMatchIterator, reader)
+	defer func() {
+		err = multierr.Append(err, iter.Close())
+	}()
 	list = roaring.NewPostingList()
 	for iter.Next() {
-		err = multierr.Append(err, list.Union(iter.Val().Value))
+		docID, _ := iter.Val()
+		list.Insert(docID)
 	}
-	err = multierr.Append(err, iter.Close())
 	return list, err
 }
 
@@ -245,7 +258,9 @@ func (s *store) Match(fieldKey index.FieldKey, matches []string) (posting.List,
 	analyzer := analyzers[fieldKey.Analyzer]
 	fk := fieldKey.MarshalIndexRule()
 	query := bluge.NewBooleanQuery()
-	query.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).SetField(seriesIDField))
+	if fieldKey.HasSeriesID() {
+		query.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).SetField(seriesIDField))
+	}
 	for _, m := range matches {
 		query.AddMust(bluge.NewMatchQuery(m).SetField(fk).
 			SetAnalyzer(analyzer))
@@ -254,31 +269,45 @@ func (s *store) Match(fieldKey index.FieldKey, matches []string) (posting.List,
 	if err != nil {
 		return nil, err
 	}
-	iter := newBlugeMatchIterator(documentMatchIterator, fk, false)
+	iter := newBlugeMatchIterator(documentMatchIterator, reader)
+	defer func() {
+		err = multierr.Append(err, iter.Close())
+	}()
 	list := roaring.NewPostingList()
 	for iter.Next() {
-		err = multierr.Append(err, list.Union(iter.Val().Value))
+		docID, _ := iter.Val()
+		list.Insert(docID)
 	}
-	err = multierr.Append(err, iter.Close())
 	return list, err
 }
 
 func (s *store) Range(fieldKey index.FieldKey, opts index.RangeOpts) (list posting.List, err error) {
-	iter, err := s.Iterator(fieldKey, opts, modelv1.Sort_SORT_ASC)
+	iter, err := s.Iterator(fieldKey, opts, modelv1.Sort_SORT_ASC, defaultRangePreloadSize)
 	if err != nil {
 		return roaring.DummyPostingList, err
 	}
 	list = roaring.NewPostingList()
 	for iter.Next() {
-		err = multierr.Append(err, list.Union(iter.Val().Value))
+		docID, _ := iter.Val()
+		list.Insert(docID)
 	}
 	err = multierr.Append(err, iter.Close())
 	return
 }
 
+func (s *store) SizeOnDisk() int64 {
+	_, bytes := s.writer.DirectoryStats()
+	return int64(bytes)
+}
+
 func (s *store) run() {
 	go func() {
-		defer s.closer.Done()
+		defer func() {
+			if err := s.writer.Close(); err != nil {
+				s.errClosing.Store(&err)
+			}
+			s.closer.Done()
+		}()
 		size := 0
 		batch := bluge.NewBatch()
 		flush := func() {
@@ -291,49 +320,76 @@ func (s *store) run() {
 			batch.Reset()
 			size = 0
 		}
+		defer flush()
 		var docIDBuffer bytes.Buffer
 		for {
 			timer := time.NewTimer(s.batchInterval)
 			select {
 			case <-s.closer.CloseNotify():
+				timer.Stop()
 				return
 			case event, more := <-s.ch:
 				if !more {
+					timer.Stop()
 					return
 				}
 				switch d := event.(type) {
 				case flushEvent:
 					flush()
 					close(d.onComplete)
-				case doc:
-					// TODO: generate a segment directly.
-					fk := d.fields[0].Key
-					docIDBuffer.Reset()
-					docIDBuffer.Write(fk.SeriesID.Marshal())
-					docIDBuffer.Write(convert.Uint64ToBytes(uint64(d.itemID)))
-					doc := bluge.NewDocument(docIDBuffer.String())
-					toAddSeriesIDField := false
-					for _, f := range d.fields {
-						if f.Key.Analyzer == databasev1.IndexRule_ANALYZER_UNSPECIFIED {
-							doc.AddField(bluge.NewKeywordFieldBytes(f.Key.MarshalIndexRule(), f.Marshal()).StoreValue().Sortable())
-						} else {
-							toAddSeriesIDField = true
-							doc.AddField(bluge.NewKeywordFieldBytes(f.Key.MarshalIndexRule(), f.Term).StoreValue().Sortable().
-								WithAnalyzer(analyzers[f.Key.Analyzer]))
+				case index.Document, index.Batch:
+					var docs []index.Document
+					var isBatch bool
+					var applied chan struct{}
+					switch v := d.(type) {
+					case index.Document:
+						docs = []index.Document{v}
+					case index.Batch:
+						docs = v.Documents
+						applied = v.Applied
+						isBatch = true
+					}
+
+					for _, d := range docs {
+						// TODO: generate a segment directly.
+						var fk *index.FieldKey
+						if len(d.Fields) > 0 {
+							fk = &d.Fields[0].Key
 						}
+						docIDBuffer.Reset()
+						if fk != nil && fk.HasSeriesID() {
+							docIDBuffer.Write(fk.SeriesID.Marshal())
+						}
+						docIDBuffer.Write(convert.Uint64ToBytes(d.DocID))
+						doc := bluge.NewDocument(docIDBuffer.String())
+						for _, f := range d.Fields {
+							tf := bluge.NewKeywordFieldBytes(f.Key.MarshalIndexRule(), f.Term).Sortable()
+							if f.Key.Analyzer != databasev1.IndexRule_ANALYZER_UNSPECIFIED {
+								tf = tf.WithAnalyzer(analyzers[f.Key.Analyzer])
+							}
+							doc.AddField(tf)
+						}
+
+						if d.EntityValues != nil {
+							doc.AddField(bluge.NewKeywordFieldBytes(entityField, d.EntityValues).StoreValue())
+						} else if fk != nil && fk.HasSeriesID() {
+							doc.AddField(bluge.NewKeywordFieldBytes(seriesIDField, fk.SeriesID.Marshal()))
+						}
+
+						size++
+						batch.Update(doc.ID(), doc)
 					}
-					if toAddSeriesIDField {
-						doc.AddField(bluge.NewKeywordFieldBytes(seriesIDField, fk.SeriesID.Marshal()))
-					}
-					size++
-					batch.Update(doc.ID(), doc)
-					if size >= batchSize {
+					if isBatch || size >= batchSize {
 						flush()
+						if applied != nil {
+							close(applied)
+						}
 					}
 				}
 			case <-timer.C:
 				flush()
 			}
+			timer.Stop()
 		}
 	}()
 }
@@ -352,105 +408,59 @@ func (s *store) flush() {
 }
 
 type blugeMatchIterator struct {
-	delegated        search.DocumentMatchIterator
-	err              error
-	current          *index.PostingValue
-	agg              *index.PostingValue
-	fieldKey         string
-	shouldDecodeTerm bool
-	closed           bool
+	delegated search.DocumentMatchIterator
+	err       error
+	closer    io.Closer
+	docID     uint64
+	seriesID  common.SeriesID
 }
 
-func newBlugeMatchIterator(delegated search.DocumentMatchIterator, fieldKey string, shouldDecodeTerm bool) blugeMatchIterator {
+func newBlugeMatchIterator(delegated search.DocumentMatchIterator, closer io.Closer) blugeMatchIterator {
 	return blugeMatchIterator{
-		delegated:        delegated,
-		fieldKey:         fieldKey,
-		shouldDecodeTerm: shouldDecodeTerm,
+		delegated: delegated,
+		closer:    closer,
 	}
 }
 
 func (bmi *blugeMatchIterator) Next() bool {
-	if bmi.err != nil || bmi.closed {
-		return false
-	}
-	//revive:disable:empty-block
-	for bmi.nextTerm() {
-	}
-	//revive:enable:empty-block
-	if bmi.err != nil || bmi.closed {
-		return false
-	}
-	return true
-}
-
-func (bmi *blugeMatchIterator) nextTerm() bool {
 	var match *search.DocumentMatch
 	match, bmi.err = bmi.delegated.Next()
 	if bmi.err != nil {
 		return false
 	}
 	if match == nil {
-		if bmi.agg == nil {
-			bmi.closed = true
-		} else {
-			bmi.current = bmi.agg
-			bmi.agg = nil
-		}
+		bmi.err = io.EOF
 		return false
 	}
-	i := 0
-	var itemID common.ItemID
-	var term []byte
 	bmi.err = match.VisitStoredFields(func(field string, value []byte) bool {
-		if field == docID {
-			// value = seriesID(8bytes)+itemID(8bytes)
-			id := convert.BytesToUint64(value[8:])
-			itemID = common.ItemID(id)
-			i++
-		}
-		if field == bmi.fieldKey {
-			v := y.Copy(value)
-			if bmi.shouldDecodeTerm {
-				term = index.UnmarshalTerm(v)
-			} else {
-				term = v
+		if field == docIDField {
+			if len(value) == 8 {
+				bmi.docID = convert.BytesToUint64(value)
+			} else if len(value) == 16 {
+				// value = seriesID(8bytes)+docID(8bytes)
+				bmi.docID = convert.BytesToUint64(value[8:])
+				bmi.seriesID = common.SeriesID(convert.BytesToUint64(value[:8]))
 			}
-			i++
 		}
-		return i < 2
+		return true
 	})
-	if i != 2 {
-		// ignore invalid data
-		// TODO: add metric to cumulate ignored docs
-		return true
-	}
-	if bmi.err != nil {
-		return false
-	}
-	if bmi.agg == nil {
-		bmi.agg = &index.PostingValue{
-			Term:  term,
-			Value: roaring.NewPostingListWithInitialData(uint64(itemID)),
-		}
-		return true
-	}
-	if bytes.Equal(bmi.agg.Term, term) {
-		bmi.agg.Value.Insert(itemID)
-		return true
-	}
-	bmi.current = bmi.agg
-	bmi.agg = &index.PostingValue{
-		Term:  term,
-		Value: roaring.NewPostingListWithInitialData(uint64(itemID)),
-	}
-	return false
+	return bmi.err == nil
 }
 
-func (bmi *blugeMatchIterator) Val() *index.PostingValue {
-	return bmi.current
+func (bmi *blugeMatchIterator) Val() (uint64, common.SeriesID) {
+	return bmi.docID, bmi.seriesID
 }
 
 func (bmi *blugeMatchIterator) Close() error {
-	bmi.closed = true
-	return bmi.err
+	if bmi.closer == nil {
+		if errors.Is(bmi.err, io.EOF) {
+			return nil
+		}
+		return bmi.err
+	}
+	err := bmi.closer.Close()
+	if errors.Is(bmi.err, io.EOF) {
+		return err
+	}
+	return errors.Join(bmi.err, bmi.closer.Close())
 }
