@@ -19,13 +19,13 @@
 package testcases
 
 import (
+	"fmt"
 	"sort"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/apache/skywalking-banyandb/api/common"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/index"
@@ -34,6 +34,7 @@ import (
 )
 
 var duration = index.FieldKey{
+	SeriesID: 998,
 	// duration
 	IndexRuleID: 3,
 }
@@ -52,14 +53,11 @@ type args struct {
 }
 
 type result struct {
-	items []int
-	key   int
+	items []uint64
 }
 
 // RunDuration executes duration related cases.
 func RunDuration(t *testing.T, data map[int]posting.List, store SimpleStore) {
-	tester := assert.New(t)
-	is := require.New(t)
 	tests := []struct {
 		name string
 		want []int
@@ -263,9 +261,34 @@ func RunDuration(t *testing.T, data map[int]posting.List, store SimpleStore) {
 			},
 		},
 	}
-	for _, tt := range tests {
+	preLoadSizes := []int{7, 20, 50}
+	allTests := make([]struct {
+		name        string
+		want        []int
+		args        args
+		preloadSize int
+	}, 0, len(tests)*len(preLoadSizes))
+
+	for _, size := range preLoadSizes {
+		for _, t := range tests {
+			allTests = append(allTests, struct {
+				name        string
+				want        []int
+				args        args
+				preloadSize int
+			}{
+				name:        t.name + " preLoadSize " + fmt.Sprint(size),
+				want:        t.want,
+				preloadSize: size,
+				args:        t.args,
+			})
+		}
+	}
+	for _, tt := range allTests {
 		t.Run(tt.name, func(t *testing.T) {
-			iter, err := store.Iterator(tt.args.fieldKey, tt.args.termRange, tt.args.orderType)
+			tester := assert.New(t)
+			is := require.New(t)
+			iter, err := store.Iterator(tt.args.fieldKey, tt.args.termRange, tt.args.orderType, tt.preloadSize)
 			is.NoError(err)
 			if iter == nil {
 				tester.Empty(tt.want)
@@ -278,40 +301,24 @@ func RunDuration(t *testing.T, data map[int]posting.List, store SimpleStore) {
 				}
 			}()
 			is.NotNil(iter)
-			got := make([]result, 0)
+			var got result
 			for iter.Next() {
-				got = append(got, result{
-					key:   int(convert.BytesToInt64(iter.Val().Term)),
-					items: toArray(iter.Val().Value),
-				})
+				docID, _ := iter.Val()
+				got.items = append(got.items, docID)
 			}
 			for i := 0; i < 10; i++ {
 				is.False(iter.Next())
 			}
-			wants := make([]result, 0, len(tt.want))
+			var wants result
 			for _, w := range tt.want {
-				wants = append(wants, result{
-					key:   w,
-					items: toArray(data[w]),
-				})
+				pl := data[w]
+				wants.items = append(wants.items, pl.ToSlice()...)
 			}
 			tester.Equal(wants, got, tt.name)
 		})
 	}
 }
 
-func toArray(list posting.List) []int {
-	ints := make([]int, 0, list.Len())
-	iter := list.Iterator()
-	defer func(iter posting.Iterator) {
-		_ = iter.Close()
-	}(iter)
-	for iter.Next() {
-		ints = append(ints, int(iter.Current()))
-	}
-	return ints
-}
-
 // SetUpDuration initializes data for testing duration related cases.
 func SetUpDuration(t *assert.Assertions, store index.Writer) map[int]posting.List {
 	r := map[int]posting.List{
@@ -331,7 +338,7 @@ func setUpPartialDuration(t *assert.Assertions, store index.Writer, r map[int]po
 	}
 	sort.Ints(idx)
 	for i := 100; i < 200; i++ {
-		id := common.ItemID(i)
+		id := uint64(i)
 		for i2, term := range idx {
 			if i%len(idx) != i2 || r[term] == nil {
 				continue
