@@ -22,13 +22,13 @@ import (
 
 	"github.com/stretchr/testify/assert"
 
-	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting/roaring"
 )
 
 var serviceName = index.FieldKey{
+	SeriesID: 887,
 	// http_method
 	IndexRuleID: 6,
 }
@@ -72,7 +72,7 @@ func RunServiceName(t *testing.T, store SimpleStore) {
 		},
 	}
 	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
+		t.Run(tt.name, func(_ *testing.T) {
 			list, err := store.MatchTerms(tt.arg)
 			if tt.wantErr {
 				tester.Error(err)
@@ -92,12 +92,12 @@ func SetUp(t *assert.Assertions, store SimpleStore) {
 			t.NoError(store.Write([]index.Field{{
 				Key:  serviceName,
 				Term: []byte("gateway"),
-			}}, common.ItemID(i)))
+			}}, uint64(i)))
 		} else {
 			t.NoError(store.Write([]index.Field{{
 				Key:  serviceName,
 				Term: []byte("webpage"),
-			}}, common.ItemID(i)))
+			}}, uint64(i)))
 		}
 	}
 }
