@@ -28,30 +28,48 @@ import (
 
 	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/api/data"
+	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	measurev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/measure/v1"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
+	"github.com/apache/skywalking-banyandb/banyand/queue"
+	"github.com/apache/skywalking-banyandb/pkg/accesslog"
 	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
+	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 type measureService struct {
 	measurev1.UnimplementedMeasureServiceServer
 	*discoveryService
-	sampled *logger.Logger
+	sampled            *logger.Logger
+	ingestionAccessLog accesslog.Log
+	pipeline           queue.Client
+	broadcaster        queue.Client
+	writeTimeout       time.Duration
 }
 
 func (ms *measureService) setLogger(log *logger.Logger) {
 	ms.sampled = log.Sampled(10)
 }
 
+func (ms *measureService) activeIngestionAccessLog(root string) (err error) {
+	if ms.ingestionAccessLog, err = accesslog.
+		NewFileLog(root, "measure-ingest-%s", 10*time.Minute, ms.log); err != nil {
+		return err
+	}
+	return nil
+}
+
 func (ms *measureService) Write(measure measurev1.MeasureService_WriteServer) error {
-	reply := func(measure measurev1.MeasureService_WriteServer, logger *logger.Logger) {
-		if errResp := measure.Send(&measurev1.WriteResponse{}); errResp != nil {
-			logger.Err(errResp).Msg("failed to send response")
+	reply := func(metadata *commonv1.Metadata, status modelv1.Status, messageId uint64, measure measurev1.MeasureService_WriteServer, logger *logger.Logger) {
+		if errResp := measure.Send(&measurev1.WriteResponse{Metadata: metadata, Status: status, MessageId: messageId}); errResp != nil {
+			logger.Debug().Err(errResp).Msg("failed to send response")
 		}
 	}
 	ctx := measure.Context()
+	publisher := ms.pipeline.NewBatchPublisher(ms.writeTimeout)
+	defer publisher.Close()
 	for {
 		select {
 		case <-ctx.Done():
@@ -64,34 +82,58 @@ func (ms *measureService) Write(measure measurev1.MeasureService_WriteServer) er
 		}
 		if err != nil {
 			ms.sampled.Error().Err(err).Stringer("written", writeRequest).Msg("failed to receive message")
-			reply(measure, ms.sampled)
-			continue
+			return err
 		}
 		if errTime := timestamp.CheckPb(writeRequest.DataPoint.Timestamp); errTime != nil {
 			ms.sampled.Error().Err(errTime).Stringer("written", writeRequest).Msg("the data point time is invalid")
-			reply(measure, ms.sampled)
+			reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_INVALID_TIMESTAMP, writeRequest.GetMessageId(), measure, ms.sampled)
 			continue
 		}
+		if writeRequest.Metadata.ModRevision > 0 {
+			measureCache, existed := ms.entityRepo.getLocator(getID(writeRequest.GetMetadata()))
+			if !existed {
+				ms.sampled.Error().Err(err).Stringer("written", writeRequest).Msg("failed to measure schema not found")
+				reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_NOT_FOUND, writeRequest.GetMessageId(), measure, ms.sampled)
+				continue
+			}
+			if writeRequest.Metadata.ModRevision != measureCache.ModRevision {
+				ms.sampled.Error().Stringer("written", writeRequest).Msg("the measure schema is expired")
+				reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_EXPIRED_SCHEMA, writeRequest.GetMessageId(), measure, ms.sampled)
+				continue
+			}
+		}
 		entity, tagValues, shardID, err := ms.navigate(writeRequest.GetMetadata(), writeRequest.GetDataPoint().GetTagFamilies())
 		if err != nil {
 			ms.sampled.Error().Err(err).RawJSON("written", logger.Proto(writeRequest)).Msg("failed to navigate to the write target")
-			reply(measure, ms.sampled)
+			reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_INTERNAL_ERROR, writeRequest.GetMessageId(), measure, ms.sampled)
 			continue
 		}
+		if ms.ingestionAccessLog != nil {
+			if errAccessLog := ms.ingestionAccessLog.Write(writeRequest); errAccessLog != nil {
+				ms.sampled.Error().Err(errAccessLog).RawJSON("written", logger.Proto(writeRequest)).Msg("failed to write access log")
+			}
+		}
 		iwr := &measurev1.InternalWriteRequest{
 			Request:    writeRequest,
 			ShardId:    uint32(shardID),
-			SeriesHash: tsdb.HashEntity(entity),
+			SeriesHash: pbv1.HashEntity(entity),
+			// TODO: remove the first value (measure name) of tagValues
+			EntityValues: tagValues[1:].Encode(),
 		}
-		if ms.log.Debug().Enabled() {
-			iwr.EntityValues = tagValues.Encode()
+		nodeID, errPickNode := ms.nodeRegistry.Locate(writeRequest.GetMetadata().GetGroup(), writeRequest.GetMetadata().GetName(), uint32(shardID))
+		if errPickNode != nil {
+			ms.sampled.Error().Err(errPickNode).RawJSON("written", logger.Proto(writeRequest)).Msg("failed to pick an available node")
+			reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_INTERNAL_ERROR, writeRequest.GetMessageId(), measure, ms.sampled)
+			continue
 		}
-		message := bus.NewMessage(bus.MessageID(time.Now().UnixNano()), iwr)
-		_, errWritePub := ms.pipeline.Publish(data.TopicMeasureWrite, message)
+		message := bus.NewBatchMessageWithNode(bus.MessageID(time.Now().UnixNano()), nodeID, iwr)
+		_, errWritePub := publisher.Publish(data.TopicMeasureWrite, message)
 		if errWritePub != nil {
-			ms.sampled.Error().Err(errWritePub).RawJSON("written", logger.Proto(writeRequest)).Msg("failed to send a message")
+			ms.sampled.Error().Err(errWritePub).RawJSON("written", logger.Proto(writeRequest)).Str("nodeID", nodeID).Msg("failed to send a message")
+			reply(writeRequest.GetMetadata(), modelv1.Status_STATUS_INTERNAL_ERROR, writeRequest.GetMessageId(), measure, ms.sampled)
+			continue
 		}
-		reply(measure, ms.sampled)
+		reply(nil, modelv1.Status_STATUS_SUCCEED, writeRequest.GetMessageId(), measure, ms.sampled)
 	}
 }
 
@@ -102,7 +144,7 @@ func (ms *measureService) Query(_ context.Context, req *measurev1.QueryRequest)
 		return nil, status.Errorf(codes.InvalidArgument, "%v is invalid :%s", req.GetTimeRange(), err)
 	}
 	message := bus.NewMessage(bus.MessageID(time.Now().UnixNano()), req)
-	feat, errQuery := ms.pipeline.Publish(data.TopicMeasureQuery, message)
+	feat, errQuery := ms.broadcaster.Publish(data.TopicMeasureQuery, message)
 	if errQuery != nil {
 		return nil, errQuery
 	}
@@ -115,8 +157,8 @@ func (ms *measureService) Query(_ context.Context, req *measurev1.QueryRequest)
 	}
 	data := msg.Data()
 	switch d := data.(type) {
-	case []*measurev1.DataPoint:
-		return &measurev1.QueryResponse{DataPoints: d}, nil
+	case *measurev1.QueryResponse:
+		return d, nil
 	case common.Error:
 		return nil, errors.WithMessage(errQueryMsg, d.Msg())
 	}
@@ -129,7 +171,7 @@ func (ms *measureService) TopN(_ context.Context, topNRequest *measurev1.TopNReq
 	}
 
 	message := bus.NewMessage(bus.MessageID(time.Now().UnixNano()), topNRequest)
-	feat, errQuery := ms.pipeline.Publish(data.TopicTopNQuery, message)
+	feat, errQuery := ms.broadcaster.Publish(data.TopicTopNQuery, message)
 	if errQuery != nil {
 		return nil, errQuery
 	}
@@ -139,10 +181,14 @@ func (ms *measureService) TopN(_ context.Context, topNRequest *measurev1.TopNReq
 	}
 	data := msg.Data()
 	switch d := data.(type) {
-	case []*measurev1.TopNList:
-		return &measurev1.TopNResponse{Lists: d}, nil
+	case *measurev1.TopNResponse:
+		return d, nil
 	case common.Error:
 		return nil, errors.WithMessage(errQueryMsg, d.Msg())
 	}
 	return nil, nil
 }
+
+func (ms *measureService) Close() error {
+	return ms.ingestionAccessLog.Close()
+}
