@@ -21,71 +21,89 @@ package grpc
 import (
 	"context"
 	"net"
+	"runtime/debug"
+	"strconv"
 	"time"
 
-	grpc_validator "github.com/grpc-ecosystem/go-grpc-middleware/validator"
+	"github.com/grpc-ecosystem/go-grpc-middleware/v2/interceptors/recovery"
+	grpc_validator "github.com/grpc-ecosystem/go-grpc-middleware/v2/interceptors/validator"
 	"github.com/pkg/errors"
 	grpclib "google.golang.org/grpc"
+	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/credentials"
 	"google.golang.org/grpc/health"
 	"google.golang.org/grpc/health/grpc_health_v1"
+	"google.golang.org/grpc/status"
 
-	"github.com/apache/skywalking-banyandb/api/event"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	measurev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/measure/v1"
 	propertyv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/property/v1"
 	streamv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/stream/v1"
-	"github.com/apache/skywalking-banyandb/banyand/discovery"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
+	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
+	"github.com/apache/skywalking-banyandb/banyand/observability"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
-	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
 )
 
-const defaultRecvSize = 1024 * 1024 * 10
+const defaultRecvSize = 10 << 20
 
 var (
-	errServerCert = errors.New("invalid server cert file")
-	errServerKey  = errors.New("invalid server key file")
-	errNoAddr     = errors.New("no address")
-	errQueryMsg   = errors.New("invalid query message")
+	errServerCert        = errors.New("invalid server cert file")
+	errServerKey         = errors.New("invalid server key file")
+	errNoAddr            = errors.New("no address")
+	errQueryMsg          = errors.New("invalid query message")
+	errAccessLogRootPath = errors.New("access log root path is required")
 )
 
+// Server defines the gRPC server.
+type Server interface {
+	run.Unit
+	GetPort() *uint32
+}
+
 type server struct {
-	pipeline queue.Queue
-	creds    credentials.TransportCredentials
-	repo     discovery.ServiceRepo
-	stopCh   chan struct{}
-	*measureRegistryServer
+	creds credentials.TransportCredentials
+	*streamRegistryServer
 	log *logger.Logger
+	*indexRuleBindingRegistryServer
 	ser *grpclib.Server
 	*propertyServer
 	*topNAggregationRegistryServer
 	*groupRegistryServer
+	stopCh chan struct{}
 	*indexRuleRegistryServer
-	streamSVC  *streamService
-	measureSVC *measureService
-	*streamRegistryServer
-	*indexRuleBindingRegistryServer
-	addr           string
-	keyFile        string
-	certFile       string
-	maxRecvMsgSize int
-	tls            bool
+	*measureRegistryServer
+	streamSVC                *streamService
+	measureSVC               *measureService
+	host                     string
+	keyFile                  string
+	certFile                 string
+	accessLogRootPath        string
+	addr                     string
+	accessLogRecorders       []accessLogRecorder
+	maxRecvMsgSize           run.Bytes
+	port                     uint32
+	enableIngestionAccessLog bool
+	tls                      bool
 }
 
 // NewServer returns a new gRPC server.
-func NewServer(_ context.Context, pipeline queue.Queue, repo discovery.ServiceRepo, schemaRegistry metadata.Service) run.Unit {
-	return &server{
-		pipeline: pipeline,
-		repo:     repo,
-		streamSVC: &streamService{
-			discoveryService: newDiscoveryService(pipeline),
-		},
-		measureSVC: &measureService{
-			discoveryService: newDiscoveryService(pipeline),
-		},
+func NewServer(_ context.Context, pipeline, broadcaster queue.Client, schemaRegistry metadata.Repo, nodeRegistry NodeRegistry) Server {
+	streamSVC := &streamService{
+		discoveryService: newDiscoveryService(schema.KindStream, schemaRegistry, nodeRegistry),
+		pipeline:         pipeline,
+		broadcaster:      broadcaster,
+	}
+	measureSVC := &measureService{
+		discoveryService: newDiscoveryService(schema.KindMeasure, schemaRegistry, nodeRegistry),
+		pipeline:         pipeline,
+		broadcaster:      broadcaster,
+	}
+	s := &server{
+		streamSVC:  streamSVC,
+		measureSVC: measureSVC,
 		streamRegistryServer: &streamRegistryServer{
 			schemaRegistry: schemaRegistry,
 		},
@@ -108,37 +126,30 @@ func NewServer(_ context.Context, pipeline queue.Queue, repo discovery.ServiceRe
 			schemaRegistry: schemaRegistry,
 		},
 	}
+	s.accessLogRecorders = []accessLogRecorder{streamSVC, measureSVC}
+	return s
 }
 
-func (s *server) PreRun() error {
+func (s *server) PreRun(_ context.Context) error {
 	s.log = logger.GetLogger("liaison-grpc")
 	s.streamSVC.setLogger(s.log)
 	s.measureSVC.setLogger(s.log)
-	components := []struct {
-		discoverySVC *discoveryService
-		shardEvent   bus.Topic
-		entityEvent  bus.Topic
-	}{
-		{
-			shardEvent:   event.StreamTopicShardEvent,
-			entityEvent:  event.StreamTopicEntityEvent,
-			discoverySVC: s.streamSVC.discoveryService,
-		},
-		{
-			shardEvent:   event.MeasureTopicShardEvent,
-			entityEvent:  event.MeasureTopicEntityEvent,
-			discoverySVC: s.measureSVC.discoveryService,
-		},
+	components := []*discoveryService{
+		s.streamSVC.discoveryService,
+		s.measureSVC.discoveryService,
 	}
 	for _, c := range components {
-		c.discoverySVC.SetLogger(s.log)
-		err := s.repo.Subscribe(c.shardEvent, c.discoverySVC.shardRepo)
-		if err != nil {
+		c.SetLogger(s.log)
+		if err := c.initialize(); err != nil {
 			return err
 		}
-		err = s.repo.Subscribe(c.entityEvent, c.discoverySVC.entityRepo)
-		if err != nil {
-			return err
+	}
+
+	if s.enableIngestionAccessLog {
+		for _, alr := range s.accessLogRecorders {
+			if err := alr.activeIngestionAccessLog(s.accessLogRootPath); err != nil {
+				return err
+			}
 		}
 	}
 	return nil
@@ -148,20 +159,39 @@ func (s *server) Name() string {
 	return "grpc"
 }
 
+func (s *server) Role() databasev1.Role {
+	return databasev1.Role_ROLE_LIAISON
+}
+
+func (s *server) GetPort() *uint32 {
+	return &s.port
+}
+
 func (s *server) FlagSet() *run.FlagSet {
 	fs := run.NewFlagSet("grpc")
-	fs.IntVarP(&s.maxRecvMsgSize, "max-recv-msg-size", "", defaultRecvSize, "the size of max receiving message")
-	fs.BoolVarP(&s.tls, "tls", "", false, "connection uses TLS if true, else plain TCP")
-	fs.StringVarP(&s.certFile, "cert-file", "", "", "the TLS cert file")
-	fs.StringVarP(&s.keyFile, "key-file", "", "", "the TLS key file")
-	fs.StringVarP(&s.addr, "addr", "", ":17912", "the address of banyand listens")
+	s.maxRecvMsgSize = defaultRecvSize
+	fs.VarP(&s.maxRecvMsgSize, "max-recv-msg-size", "", "the size of max receiving message")
+	fs.BoolVar(&s.tls, "tls", false, "connection uses TLS if true, else plain TCP")
+	fs.StringVar(&s.certFile, "cert-file", "", "the TLS cert file")
+	fs.StringVar(&s.keyFile, "key-file", "", "the TLS key file")
+	fs.StringVar(&s.host, "grpc-host", "", "the host of banyand listens")
+	fs.Uint32Var(&s.port, "grpc-port", 17912, "the port of banyand listens")
+	fs.BoolVar(&s.enableIngestionAccessLog, "enable-ingestion-access-log", false, "enable ingestion access log")
+	fs.StringVar(&s.accessLogRootPath, "access-log-root-path", "", "access log root path")
+	fs.DurationVar(&s.streamSVC.writeTimeout, "stream-write-timeout", 15*time.Second, "stream write timeout")
+	fs.DurationVar(&s.measureSVC.writeTimeout, "measure-write-timeout", 15*time.Second, "measure write timeout")
 	return fs
 }
 
 func (s *server) Validate() error {
-	if s.addr == "" {
+	s.addr = net.JoinHostPort(s.host, strconv.FormatUint(uint64(s.port), 10))
+	if s.addr == ":" {
 		return errNoAddr
 	}
+	if s.enableIngestionAccessLog && s.accessLogRootPath == "" {
+		return errAccessLogRootPath
+	}
+	observability.UpdateAddress("grpc", s.addr)
 	if !s.tls {
 		return nil
 	}
@@ -184,9 +214,31 @@ func (s *server) Serve() run.StopNotify {
 	if s.tls {
 		opts = []grpclib.ServerOption{grpclib.Creds(s.creds)}
 	}
-	opts = append(opts, grpclib.MaxRecvMsgSize(s.maxRecvMsgSize),
-		grpclib.UnaryInterceptor(grpc_validator.UnaryServerInterceptor()),
-		grpclib.StreamInterceptor(grpc_validator.StreamServerInterceptor()),
+	grpcPanicRecoveryHandler := func(p any) (err error) {
+		s.log.Error().Interface("panic", p).Str("stack", string(debug.Stack())).Msg("recovered from panic")
+
+		return status.Errorf(codes.Internal, "%s", p)
+	}
+
+	unaryMetrics, streamMetrics := observability.MetricsServerInterceptor()
+	streamChain := []grpclib.StreamServerInterceptor{
+		grpc_validator.StreamServerInterceptor(),
+		recovery.StreamServerInterceptor(recovery.WithRecoveryHandler(grpcPanicRecoveryHandler)),
+	}
+	if streamMetrics != nil {
+		streamChain = append(streamChain, streamMetrics)
+	}
+	unaryChain := []grpclib.UnaryServerInterceptor{
+		grpc_validator.UnaryServerInterceptor(),
+		recovery.UnaryServerInterceptor(recovery.WithRecoveryHandler(grpcPanicRecoveryHandler)),
+	}
+	if unaryMetrics != nil {
+		unaryChain = append(unaryChain, unaryMetrics)
+	}
+
+	opts = append(opts, grpclib.MaxRecvMsgSize(int(s.maxRecvMsgSize)),
+		grpclib.ChainUnaryInterceptor(unaryChain...),
+		grpclib.ChainStreamInterceptor(streamChain...),
 	)
 	s.ser = grpclib.NewServer(opts...)
 
@@ -199,6 +251,7 @@ func (s *server) Serve() run.StopNotify {
 	databasev1.RegisterStreamRegistryServiceServer(s.ser, s.streamRegistryServer)
 	databasev1.RegisterMeasureRegistryServiceServer(s.ser, s.measureRegistryServer)
 	propertyv1.RegisterPropertyServiceServer(s.ser, s.propertyServer)
+	databasev1.RegisterTopNAggregationRegistryServiceServer(s.ser, s.topNAggregationRegistryServer)
 	grpc_health_v1.RegisterHealthServer(s.ser, health.NewServer())
 
 	s.stopCh = make(chan struct{})
@@ -224,6 +277,11 @@ func (s *server) GracefulStop() {
 	stopped := make(chan struct{})
 	go func() {
 		s.ser.GracefulStop()
+		if s.enableIngestionAccessLog {
+			for _, alr := range s.accessLogRecorders {
+				_ = alr.Close()
+			}
+		}
 		close(stopped)
 	}()
 
@@ -237,3 +295,8 @@ func (s *server) GracefulStop() {
 		s.log.Info().Msg("stopped gracefully")
 	}
 }
+
+type accessLogRecorder interface {
+	activeIngestionAccessLog(root string) error
+	Close() error
+}
