@@ -19,43 +19,86 @@ package measure
 
 import (
 	"context"
+	"fmt"
+	"io"
 	"path"
 	"time"
 
+	"github.com/pkg/errors"
+
 	"github.com/apache/skywalking-banyandb/api/common"
-	"github.com/apache/skywalking-banyandb/api/event"
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
-	"github.com/apache/skywalking-banyandb/banyand/discovery"
+	"github.com/apache/skywalking-banyandb/api/validate"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
+	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
-	pb_v1 "github.com/apache/skywalking-banyandb/pkg/pb/v1/tsdb"
 	resourceSchema "github.com/apache/skywalking-banyandb/pkg/schema"
 )
 
+// SchemaService allows querying schema information.
+type SchemaService interface {
+	Query
+	Close()
+}
 type schemaRepo struct {
 	resourceSchema.Repository
 	l        *logger.Logger
 	metadata metadata.Repo
 }
 
-func newSchemaRepo(path string, metadata metadata.Repo, repo discovery.ServiceRepo,
-	dbOpts tsdb.DatabaseOpts, l *logger.Logger,
-) schemaRepo {
-	return schemaRepo{
-		l:        l,
-		metadata: metadata,
+func newSchemaRepo(path string, svc *service) schemaRepo {
+	sr := schemaRepo{
+		l:        svc.l,
+		metadata: svc.metadata,
 		Repository: resourceSchema.NewRepository(
-			metadata,
-			repo,
-			l,
-			newSupplier(path, metadata, dbOpts, l),
-			event.MeasureTopicShardEvent,
-			event.MeasureTopicEntityEvent,
+			svc.metadata,
+			svc.l,
+			newSupplier(path, svc),
 		),
 	}
+	sr.start()
+	return sr
+}
+
+// NewPortableRepository creates a new portable repository.
+func NewPortableRepository(metadata metadata.Repo, l *logger.Logger) SchemaService {
+	r := &schemaRepo{
+		l:        l,
+		metadata: metadata,
+		Repository: resourceSchema.NewPortableRepository(
+			metadata,
+			l,
+			newPortableSupplier(metadata, l),
+		),
+	}
+	r.start()
+	return r
+}
+
+func (sr *schemaRepo) start() {
+	sr.Watcher()
+	sr.metadata.
+		RegisterHandler("measure", schema.KindGroup|schema.KindMeasure|schema.KindIndexRuleBinding|schema.KindIndexRule|schema.KindTopNAggregation,
+			sr)
+}
+
+func (sr *schemaRepo) Measure(metadata *commonv1.Metadata) (Measure, error) {
+	sm, ok := sr.loadMeasure(metadata)
+	if !ok {
+		return nil, errors.WithStack(ErrMeasureNotExist)
+	}
+	return sm, nil
+}
+
+func (sr *schemaRepo) OnInit(kinds []schema.Kind) (bool, []int64) {
+	if len(kinds) != 5 {
+		logger.Panicf("unexpected kinds: %v", kinds)
+		return false, nil
+	}
+	return true, sr.Repository.Init(schema.KindMeasure)
 }
 
 func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
@@ -65,16 +108,24 @@ func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
 		if g.Catalog != commonv1.Catalog_CATALOG_MEASURE {
 			return
 		}
+		if err := validate.GroupForStreamOrMeasure(g); err != nil {
+			sr.l.Warn().Err(err).Msg("group is ignored")
+			return
+		}
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventAddOrUpdate,
 			Kind:     resourceSchema.EventKindGroup,
-			Metadata: g.GetMetadata(),
+			Metadata: g,
 		})
 	case schema.KindMeasure:
+		if err := validate.Measure(metadata.Spec.(*databasev1.Measure)); err != nil {
+			sr.l.Warn().Err(err).Msg("measure is ignored")
+			return
+		}
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventAddOrUpdate,
 			Kind:     resourceSchema.EventKindResource,
-			Metadata: metadata.Spec.(*databasev1.Measure).GetMetadata(),
+			Metadata: metadata.Spec.(*databasev1.Measure),
 		})
 	case schema.KindIndexRuleBinding:
 		irb, ok := metadata.Spec.(*databasev1.IndexRuleBinding)
@@ -82,6 +133,10 @@ func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
 			sr.l.Warn().Msg("fail to convert message to IndexRuleBinding")
 			return
 		}
+		if err := validate.IndexRuleBinding(irb); err != nil {
+			sr.l.Warn().Err(err).Msg("index rule binding is ignored")
+			return
+		}
 		if irb.GetSubject().Catalog == commonv1.Catalog_CATALOG_MEASURE {
 			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 			stm, err := sr.metadata.MeasureRegistry().GetMeasure(ctx, &commonv1.Metadata{
@@ -90,35 +145,41 @@ func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
 			})
 			cancel()
 			if err != nil {
-				sr.l.Error().Err(err).Msg("fail to get subject")
 				return
 			}
 			sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 				Typ:      resourceSchema.EventAddOrUpdate,
 				Kind:     resourceSchema.EventKindResource,
-				Metadata: stm.GetMetadata(),
+				Metadata: stm,
 			})
 		}
 	case schema.KindIndexRule:
+		if err := validate.IndexRule(metadata.Spec.(*databasev1.IndexRule)); err != nil {
+			sr.l.Warn().Err(err).Msg("index rule is ignored")
+			return
+		}
 		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 		defer cancel()
 		subjects, err := sr.metadata.Subjects(ctx, metadata.Spec.(*databasev1.IndexRule), commonv1.Catalog_CATALOG_MEASURE)
 		if err != nil {
-			sr.l.Error().Err(err).Msg("fail to get subjects(measure)")
 			return
 		}
 		for _, sub := range subjects {
 			sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 				Typ:      resourceSchema.EventAddOrUpdate,
 				Kind:     resourceSchema.EventKindResource,
-				Metadata: sub.(*databasev1.Measure).GetMetadata(),
+				Metadata: sub.(*databasev1.Measure),
 			})
 		}
 	case schema.KindTopNAggregation:
+		if err := validate.TopNAggregation(metadata.Spec.(*databasev1.TopNAggregation)); err != nil {
+			sr.l.Warn().Err(err).Msg("topNAggregation is ignored")
+			return
+		}
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventAddOrUpdate,
-			Kind:     resourceSchema.EventKindResource,
-			Metadata: metadata.Spec.(*databasev1.TopNAggregation).GetSourceMeasure(),
+			Kind:     resourceSchema.EventKindTopNAgg,
+			Metadata: metadata.Spec.(*databasev1.TopNAggregation),
 		})
 	default:
 	}
@@ -134,13 +195,13 @@ func (sr *schemaRepo) OnDelete(metadata schema.Metadata) {
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventDelete,
 			Kind:     resourceSchema.EventKindGroup,
-			Metadata: g.GetMetadata(),
+			Metadata: g,
 		})
 	case schema.KindMeasure:
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventDelete,
 			Kind:     resourceSchema.EventKindResource,
-			Metadata: metadata.Spec.(*databasev1.Measure).GetMetadata(),
+			Metadata: metadata.Spec.(*databasev1.Measure),
 		})
 	case schema.KindIndexRuleBinding:
 		if metadata.Spec.(*databasev1.IndexRuleBinding).GetSubject().Catalog == commonv1.Catalog_CATALOG_MEASURE {
@@ -151,23 +212,21 @@ func (sr *schemaRepo) OnDelete(metadata schema.Metadata) {
 				Group: metadata.Group,
 			})
 			if err != nil {
-				sr.l.Error().Err(err).Msg("fail to get subject")
 				return
 			}
 			// we should update instead of delete
 			sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 				Typ:      resourceSchema.EventAddOrUpdate,
 				Kind:     resourceSchema.EventKindResource,
-				Metadata: m.GetMetadata(),
+				Metadata: m,
 			})
 		}
 	case schema.KindIndexRule:
 	case schema.KindTopNAggregation:
-		// we should update instead of delete
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventAddOrUpdate,
-			Kind:     resourceSchema.EventKindResource,
-			Metadata: metadata.Spec.(*databasev1.TopNAggregation).GetSourceMeasure(),
+			Kind:     resourceSchema.EventKindTopNAgg,
+			Metadata: metadata.Spec.(*databasev1.TopNAggregation),
 		})
 	default:
 	}
@@ -178,35 +237,49 @@ func (sr *schemaRepo) loadMeasure(metadata *commonv1.Metadata) (*measure, bool)
 	if !ok {
 		return nil, false
 	}
-	s, ok := r.(*measure)
+	s, ok := r.Delegated().(*measure)
 	return s, ok
 }
 
+func (sr *schemaRepo) loadTSDB(groupName string) (storage.TSDB[*tsTable, option], error) {
+	g, ok := sr.LoadGroup(groupName)
+	if !ok {
+		return nil, fmt.Errorf("group %s not found", groupName)
+	}
+	db := g.SupplyTSDB()
+	if db == nil {
+		return nil, fmt.Errorf("group %s not found", groupName)
+	}
+	return db.(storage.TSDB[*tsTable, option]), nil
+}
+
 var _ resourceSchema.ResourceSupplier = (*supplier)(nil)
 
 type supplier struct {
 	metadata metadata.Repo
+	pipeline queue.Queue
+	option   option
 	l        *logger.Logger
 	path     string
-	dbOpts   tsdb.DatabaseOpts
 }
 
-func newSupplier(path string, metadata metadata.Repo, dbOpts tsdb.DatabaseOpts, l *logger.Logger) *supplier {
+func newSupplier(path string, svc *service) *supplier {
 	return &supplier{
 		path:     path,
-		dbOpts:   dbOpts,
-		metadata: metadata,
-		l:        l,
+		metadata: svc.metadata,
+		l:        svc.l,
+		pipeline: svc.localPipeline,
+		option:   svc.option,
 	}
 }
 
-func (s *supplier) OpenResource(shardNum uint32, db tsdb.Supplier, spec resourceSchema.ResourceSpec) (resourceSchema.Resource, error) {
-	measureSchema := spec.Schema.(*databasev1.Measure)
-	return openMeasure(shardNum, db, measureSpec{
+func (s *supplier) OpenResource(shardNum uint32, supplier resourceSchema.Supplier, spec resourceSchema.Resource) (io.Closer, error) {
+	measureSchema := spec.Schema().(*databasev1.Measure)
+	return openMeasure(shardNum, supplier, measureSpec{
 		schema:           measureSchema,
-		indexRules:       spec.IndexRules,
-		topNAggregations: spec.Aggregations,
-	}, s.l)
+		indexRules:       spec.IndexRules(),
+		topNAggregations: spec.TopN(),
+	}, s.l, s.pipeline)
 }
 
 func (s *supplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.ResourceSchema, error) {
@@ -215,35 +288,53 @@ func (s *supplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.Resourc
 	return s.metadata.MeasureRegistry().GetMeasure(ctx, md)
 }
 
-func (s *supplier) OpenDB(groupSchema *commonv1.Group) (tsdb.Database, error) {
-	opts := s.dbOpts
-	opts.ShardNum = groupSchema.ResourceOpts.ShardNum
-	opts.Location = path.Join(s.path, groupSchema.Metadata.Name)
+func (s *supplier) OpenDB(groupSchema *commonv1.Group) (io.Closer, error) {
+	opts := storage.TSDBOpts[*tsTable, option]{
+		ShardNum:                       groupSchema.ResourceOpts.ShardNum,
+		Location:                       path.Join(s.path, groupSchema.Metadata.Name),
+		TSTableCreator:                 newTSTable,
+		SegmentInterval:                storage.MustToIntervalRule(groupSchema.ResourceOpts.SegmentInterval),
+		TTL:                            storage.MustToIntervalRule(groupSchema.ResourceOpts.Ttl),
+		Option:                         s.option,
+		SeriesIndexFlushTimeoutSeconds: s.option.flushTimeout.Nanoseconds() / int64(time.Second),
+	}
 	name := groupSchema.Metadata.Name
-	opts.EncodingMethod = tsdb.EncodingMethod{
-		EncoderPool:      newEncoderPool(name, intChunkNum, s.l),
-		DecoderPool:      newDecoderPool(name, intChunkNum, s.l),
-		ChunkSizeInBytes: intChunkSize,
-	}
-	opts.CompressionMethod = tsdb.CompressionMethod{
-		Type:             tsdb.CompressionTypeZSTD,
-		ChunkSizeInBytes: plainChunkSize,
-	}
-
-	var err error
-	if opts.BlockInterval, err = pb_v1.ToIntervalRule(groupSchema.ResourceOpts.BlockInterval); err != nil {
-		return nil, err
-	}
-	if opts.SegmentInterval, err = pb_v1.ToIntervalRule(groupSchema.ResourceOpts.SegmentInterval); err != nil {
-		return nil, err
-	}
-	if opts.TTL, err = pb_v1.ToIntervalRule(groupSchema.ResourceOpts.Ttl); err != nil {
-		return nil, err
-	}
-	return tsdb.OpenDatabase(
-		context.WithValue(context.Background(), common.PositionKey, common.Position{
-			Module:   "measure",
-			Database: name,
+	return storage.OpenTSDB(
+		common.SetPosition(context.Background(), func(p common.Position) common.Position {
+			p.Module = "measure"
+			p.Database = name
+			return p
 		}),
 		opts)
 }
+
+type portableSupplier struct {
+	metadata metadata.Repo
+	l        *logger.Logger
+}
+
+func newPortableSupplier(metadata metadata.Repo, l *logger.Logger) *portableSupplier {
+	return &portableSupplier{
+		metadata: metadata,
+		l:        l,
+	}
+}
+
+func (s *portableSupplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.ResourceSchema, error) {
+	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+	defer cancel()
+	return s.metadata.MeasureRegistry().GetMeasure(ctx, md)
+}
+
+func (*portableSupplier) OpenDB(_ *commonv1.Group) (io.Closer, error) {
+	panic("do not support open db")
+}
+
+func (s *portableSupplier) OpenResource(shardNum uint32, _ resourceSchema.Supplier, spec resourceSchema.Resource) (io.Closer, error) {
+	measureSchema := spec.Schema().(*databasev1.Measure)
+	return openMeasure(shardNum, nil, measureSpec{
+		schema:           measureSchema,
+		indexRules:       spec.IndexRules(),
+		topNAggregations: spec.TopN(),
+	}, s.l, nil)
+}
