@@ -82,7 +82,13 @@ var checkerMap = map[Kind]equalityChecker{
 			protocmp.IgnoreFields(&commonv1.Metadata{}, "id", "create_revision", "mod_revision"),
 			protocmp.Transform())
 	},
-	KindMask: func(a, b proto.Message) bool {
+	KindNode: func(a, b proto.Message) bool {
+		return cmp.Equal(a, b,
+			protocmp.IgnoreUnknown(),
+			protocmp.IgnoreFields(&databasev1.Node{}, "created_at"),
+			protocmp.Transform())
+	},
+	KindMask: func(_, _ proto.Message) bool {
 		return false
 	},
 }
