@@ -20,7 +20,6 @@ package schema
 import (
 	"context"
 	"embed"
-	"errors"
 	"fmt"
 	"os"
 	"path"
@@ -28,12 +27,12 @@ import (
 
 	"github.com/google/uuid"
 	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/protobuf/encoding/protojson"
 
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
+	"github.com/apache/skywalking-banyandb/banyand/metadata/embeddedetcd"
 	"github.com/apache/skywalking-banyandb/pkg/test"
 )
 
@@ -48,22 +47,8 @@ var (
 	streamJSON string
 	//go:embed testdata/group.json
 	groupJSON string
-
-	_ EventHandler = (*mockedEventHandler)(nil)
 )
 
-type mockedEventHandler struct {
-	mock.Mock
-}
-
-func (m *mockedEventHandler) OnAddOrUpdate(metadata Metadata) {
-	m.Called(metadata)
-}
-
-func (m *mockedEventHandler) OnDelete(metadata Metadata) {
-	m.Called(metadata)
-}
-
 func preloadSchema(e Registry) error {
 	g := &commonv1.Group{}
 	if err := protojson.Unmarshal([]byte(groupJSON), g); err != nil {
@@ -77,7 +62,7 @@ func preloadSchema(e Registry) error {
 	if err := protojson.Unmarshal([]byte(streamJSON), s); err != nil {
 		return err
 	}
-	err := e.CreateStream(context.Background(), s)
+	_, err := e.CreateStream(context.Background(), s)
 	if err != nil {
 		return err
 	}
@@ -96,7 +81,7 @@ func preloadSchema(e Registry) error {
 		return err
 	}
 	for _, entry := range entries {
-		data, err := indexRuleStore.ReadFile(indexRuleDir + "/" + entry.Name())
+		data, err := indexRuleStore.ReadFile(path.Join(indexRuleDir, entry.Name()))
 		if err != nil {
 			return err
 		}
@@ -118,30 +103,34 @@ func randomTempDir() string {
 	return path.Join(os.TempDir(), fmt.Sprintf("banyandb-embed-etcd-%s", uuid.New().String()))
 }
 
-func useRandomTempDir() RegistryOption {
-	return func(config *etcdSchemaRegistryConfig) {
-		config.rootDir = randomTempDir()
+func initServerAndRegister(t *testing.T) (Registry, func()) {
+	req := require.New(t)
+	ports, err := test.AllocateFreePorts(2)
+	if err != nil {
+		panic("fail to find free ports")
 	}
-}
-
-func useRandomPort() RegistryOption {
-	return func(config *etcdSchemaRegistryConfig) {
-		ports, err := test.AllocateFreePorts(2)
-		if err != nil {
-			panic("fail to find free ports")
-		}
-		config.listenerClientURL, config.listenerPeerURL = fmt.Sprintf("http://127.0.0.1:%d", ports[0]), fmt.Sprintf("http://127.0.0.1:%d", ports[1])
+	endpoints := []string{fmt.Sprintf("http://127.0.0.1:%d", ports[0])}
+	server, err := embeddedetcd.NewServer(
+		embeddedetcd.ConfigureListener(endpoints, []string{fmt.Sprintf("http://127.0.0.1:%d", ports[1])}),
+		embeddedetcd.RootDir(randomTempDir()))
+	req.NoError(err)
+	req.NotNil(server)
+	<-server.ReadyNotify()
+	schemaRegistry, err := NewEtcdSchemaRegistry(ConfigureServerEndpoints(endpoints))
+	req.NoError(err)
+	req.NotNil(server)
+	return schemaRegistry, func() {
+		server.Close()
+		<-server.StopNotify()
+		schemaRegistry.Close()
 	}
 }
 
 func Test_Etcd_Entity_Get(t *testing.T) {
 	tester := assert.New(t)
-	registry, err := NewEtcdSchemaRegistry(useRandomPort(), useRandomTempDir())
-	tester.NoError(err)
-	tester.NotNil(registry)
-	defer registry.Close()
-
-	err = preloadSchema(registry)
+	registry, closer := initServerAndRegister(t)
+	defer closer()
+	err := preloadSchema(registry)
 	tester.NoError(err)
 
 	tests := []struct {
@@ -153,7 +142,7 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 		{
 			name: "Get Group",
 			meta: &commonv1.Metadata{Name: "default"},
-			get: func(r Registry, meta *commonv1.Metadata) (HasMetadata, error) {
+			get: func(_ Registry, meta *commonv1.Metadata) (HasMetadata, error) {
 				stm, innerErr := registry.GetGroup(context.TODO(), meta.GetName())
 				if innerErr != nil {
 					return nil, innerErr
@@ -164,7 +153,7 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 		{
 			name: "Get Stream",
 			meta: &commonv1.Metadata{Name: "sw", Group: "default"},
-			get: func(r Registry, meta *commonv1.Metadata) (HasMetadata, error) {
+			get: func(_ Registry, meta *commonv1.Metadata) (HasMetadata, error) {
 				stm, innerErr := registry.GetStream(context.TODO(), meta)
 				if innerErr != nil {
 					return nil, innerErr
@@ -175,7 +164,7 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 		{
 			name: "Get IndexRuleBinding",
 			meta: &commonv1.Metadata{Name: "sw-index-rule-binding", Group: "default"},
-			get: func(r Registry, meta *commonv1.Metadata) (HasMetadata, error) {
+			get: func(_ Registry, meta *commonv1.Metadata) (HasMetadata, error) {
 				e, innerErr := registry.GetIndexRuleBinding(context.TODO(), meta)
 				if innerErr != nil {
 					return nil, innerErr
@@ -186,7 +175,7 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 		{
 			name: "Get IndexRule",
 			meta: &commonv1.Metadata{Name: "db.instance", Group: "default"},
-			get: func(r Registry, meta *commonv1.Metadata) (HasMetadata, error) {
+			get: func(_ Registry, meta *commonv1.Metadata) (HasMetadata, error) {
 				e, innerErr := registry.GetIndexRule(context.TODO(), meta)
 				if innerErr != nil {
 					return nil, innerErr
@@ -197,7 +186,7 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 		{
 			name: "Get unknown Measure",
 			meta: &commonv1.Metadata{Name: "unknown-stream", Group: "default"},
-			get: func(r Registry, meta *commonv1.Metadata) (HasMetadata, error) {
+			get: func(_ Registry, meta *commonv1.Metadata) (HasMetadata, error) {
 				e, innerErr := registry.GetMeasure(context.TODO(), meta)
 				if innerErr != nil {
 					return nil, innerErr
@@ -228,12 +217,10 @@ func Test_Etcd_Entity_Get(t *testing.T) {
 
 func Test_Etcd_Entity_List(t *testing.T) {
 	tester := assert.New(t)
-	registry, err := NewEtcdSchemaRegistry(useRandomPort(), useRandomTempDir())
-	tester.NoError(err)
-	tester.NotNil(registry)
-	defer registry.Close()
+	registry, closer := initServerAndRegister(t)
+	defer closer()
 
-	err = preloadSchema(registry)
+	err := preloadSchema(registry)
 	tester.NoError(err)
 
 	tests := []struct {
@@ -310,12 +297,10 @@ func Test_Etcd_Entity_List(t *testing.T) {
 
 func Test_Etcd_Delete(t *testing.T) {
 	tester := assert.New(t)
-	registry, err := NewEtcdSchemaRegistry(useRandomPort(), useRandomTempDir())
-	tester.NoError(err)
-	tester.NotNil(registry)
-	defer registry.Close()
+	registry, closer := initServerAndRegister(t)
+	defer closer()
 
-	err = preloadSchema(registry)
+	err := preloadSchema(registry)
 	tester.NoError(err)
 
 	tests := []struct {
@@ -377,24 +362,22 @@ func Test_Etcd_Delete(t *testing.T) {
 	}
 }
 
-func Test_Notify(t *testing.T) {
-	req := require.New(t)
-	registry, err := NewEtcdSchemaRegistry(useRandomPort(), useRandomTempDir())
-	req.NoError(err)
-	req.NotNil(registry)
-	defer registry.Close()
+func Test_Etcd_Entity_Update(t *testing.T) {
+	tester := assert.New(t)
+	registry, closer := initServerAndRegister(t)
+	defer closer()
 
-	err = preloadSchema(registry)
-	req.NoError(err)
+	err := preloadSchema(registry)
+	tester.NoError(err)
 
 	tests := []struct {
-		testFunc       func(context.Context, Registry) error
-		validationFunc func(*mockedEventHandler) bool
+		updateFunc     func(context.Context, Registry) error
+		validationFunc func(context.Context, Registry) bool
 		name           string
 	}{
 		{
-			name: "modify indexRule",
-			testFunc: func(ctx context.Context, r Registry) error {
+			name: "update indexRule when none metadata-id",
+			updateFunc: func(ctx context.Context, r Registry) error {
 				ir, err := r.GetIndexRule(ctx, &commonv1.Metadata{
 					Name:  "db.instance",
 					Group: "default",
@@ -402,105 +385,26 @@ func Test_Notify(t *testing.T) {
 				if err != nil {
 					return err
 				}
-
-				ir.Type = databasev1.IndexRule_TYPE_TREE
+				// reset
+				ir.Metadata.Id = 0
+				ir.Type = databasev1.IndexRule_TYPE_INVERTED
 				return r.UpdateIndexRule(ctx, ir)
 			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 1) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 0)
-			},
-		},
-		{
-			name: "modify indexRule without modification",
-			testFunc: func(ctx context.Context, r Registry) error {
+			validationFunc: func(ctx context.Context, r Registry) bool {
 				ir, err := r.GetIndexRule(ctx, &commonv1.Metadata{
 					Name:  "db.instance",
 					Group: "default",
 				})
 				if err != nil {
-					return err
+					return false
 				}
-
-				return r.UpdateIndexRule(ctx, ir)
-			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 0) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 0)
-			},
-		},
-		{
-			name: "delete indexRule",
-			testFunc: func(ctx context.Context, r Registry) error {
-				deleted, err := r.DeleteIndexRule(ctx, &commonv1.Metadata{
-					Name:  "db.instance",
-					Group: "default",
-				})
-
-				if !deleted {
-					return errors.New("fail to delete object")
+				if ir.Metadata.Id != 1 {
+					return false
 				}
-
-				return err
-			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 0) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 1)
-			},
-		},
-		{
-			name: "update indexRuleBinding",
-			testFunc: func(ctx context.Context, r Registry) error {
-				irb, err := r.GetIndexRuleBinding(ctx, &commonv1.Metadata{
-					Name:  "sw-index-rule-binding",
-					Group: "default",
-				})
-				if err != nil {
-					return err
+				if ir.Type != databasev1.IndexRule_TYPE_INVERTED {
+					return false
 				}
-
-				irb.Rules = []string{"trace_id", "duration"}
-				return r.UpdateIndexRuleBinding(ctx, irb)
-			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 1) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 0)
-			},
-		},
-		{
-			name: "update indexRuleBinding without modification",
-			testFunc: func(ctx context.Context, r Registry) error {
-				irb, err := r.GetIndexRuleBinding(ctx, &commonv1.Metadata{
-					Name:  "sw-index-rule-binding",
-					Group: "default",
-				})
-				if err != nil {
-					return err
-				}
-
-				return r.UpdateIndexRuleBinding(ctx, irb)
-			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 0) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 0)
-			},
-		},
-		{
-			name: "delete indexRuleBinding",
-			testFunc: func(ctx context.Context, r Registry) error {
-				_, err := r.DeleteIndexRuleBinding(ctx, &commonv1.Metadata{
-					Name:  "sw-index-rule-binding",
-					Group: "default",
-				})
-				if err != nil {
-					return err
-				}
-
-				return nil
-			},
-			validationFunc: func(mocked *mockedEventHandler) bool {
-				return mocked.AssertNumberOfCalls(t, "OnAddOrUpdate", 0) &&
-					mocked.AssertNumberOfCalls(t, "OnDelete", 1)
+				return true
 			},
 		},
 	}
@@ -509,15 +413,10 @@ func Test_Notify(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			req := require.New(t)
 
-			mockedObj := new(mockedEventHandler)
-			mockedObj.On("OnAddOrUpdate", mock.Anything).Return()
-			mockedObj.On("OnDelete", mock.Anything).Return()
-			registry.RegisterHandler(KindStream|KindIndexRuleBinding|KindIndexRule, mockedObj)
-
-			err := tt.testFunc(context.TODO(), registry)
+			err := tt.updateFunc(context.TODO(), registry)
 			req.NoError(err)
 
-			req.True(tt.validationFunc(mockedObj))
+			req.True(tt.validationFunc(context.TODO(), registry))
 		})
 	}
 }
