@@ -296,73 +296,6 @@ func (s *strArrLiteral) String() string {
 	return fmt.Sprintf("%v", s.arr)
 }
 
-var _ LiteralExpr = (*idLiteral)(nil)
-
-type idLiteral struct {
-	string
-}
-
-func (s *idLiteral) Bytes() [][]byte {
-	return [][]byte{[]byte(s.string)}
-}
-
-func (s *idLiteral) Compare(other LiteralExpr) (int, bool) {
-	if o, ok := other.(*idLiteral); ok {
-		return strings.Compare(s.string, o.string), true
-	}
-	return 0, false
-}
-
-func (s *idLiteral) Contains(other LiteralExpr) bool {
-	if o, ok := other.(*idLiteral); ok {
-		return s == o
-	}
-	if o, ok := other.(*strLiteral); ok {
-		return s.string == o.string
-	}
-	if o, ok := other.(*strArrLiteral); ok {
-		if len(o.arr) == 1 && o.arr[0] == s.string {
-			return true
-		}
-	}
-	return false
-}
-
-func (s *idLiteral) BelongTo(other LiteralExpr) bool {
-	if o, ok := other.(*idLiteral); ok {
-		return s == o
-	}
-	if o, ok := other.(*strLiteral); ok {
-		return s.string == o.string
-	}
-	if o, ok := other.(*strArrLiteral); ok {
-		return slices.Contains(o.arr, s.string)
-	}
-	return false
-}
-
-func (s *idLiteral) Equal(expr Expr) bool {
-	if other, ok := expr.(*idLiteral); ok {
-		return other.string == s.string
-	}
-
-	return false
-}
-
-func id(id string) LiteralExpr {
-	return &idLiteral{id}
-}
-
-func (s *idLiteral) DataType() int32 {
-	return int32(databasev1.TagType_TAG_TYPE_ID)
-}
-
-func (s *idLiteral) String() string {
-	return s.string
-}
-
-var _ LiteralExpr = (*idLiteral)(nil)
-
 type bytesLiteral struct {
 	bb []byte
 }
