@@ -28,10 +28,9 @@ import (
 	"github.com/apache/skywalking-banyandb/api/common"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
-	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting"
+	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 )
 
 var errInvalidLogicalExpression = errors.New("invalid logical expression")
@@ -46,10 +45,11 @@ type GlobalIndexError struct {
 func (g GlobalIndexError) Error() string { return g.IndexRule.String() }
 
 // BuildLocalFilter returns a new index.Filter for local indices.
-// It could parse series Path at the same time.
-func BuildLocalFilter(criteria *modelv1.Criteria, schema Schema, entityDict map[string]int, entity tsdb.Entity) (index.Filter, []tsdb.Entity, error) {
+func BuildLocalFilter(criteria *modelv1.Criteria, schema Schema, entityDict map[string]int,
+	entity []*modelv1.TagValue, mandatoryIndexRule bool,
+) (index.Filter, [][]*modelv1.TagValue, error) {
 	if criteria == nil {
-		return nil, []tsdb.Entity{entity}, nil
+		return nil, [][]*modelv1.TagValue{entity}, nil
 	}
 	switch criteria.GetExp().(type) {
 	case *modelv1.Criteria_Condition:
@@ -62,30 +62,27 @@ func BuildLocalFilter(criteria *modelv1.Criteria, schema Schema, entityDict map[
 			return nil, parsedEntity, nil
 		}
 		if ok, indexRule := schema.IndexDefined(cond.Name); ok {
-			if indexRule.Location == databasev1.IndexRule_LOCATION_GLOBAL {
-				switch cond.Op {
-				case modelv1.Condition_BINARY_OP_EQ, modelv1.Condition_BINARY_OP_IN:
-					return nil, nil, GlobalIndexError{
-						IndexRule: indexRule,
-						Expr:      expr,
-					}
-				default:
-					return nil, nil, errors.Wrapf(errUnsupportedConditionOp, "gobal index conf:%s", cond)
-				}
-			}
 			return parseCondition(cond, indexRule, expr, entity)
+		} else if mandatoryIndexRule {
+			return nil, nil, errors.Wrapf(errUnsupportedConditionOp, "mandatory index rule conf:%s", cond)
 		}
-		return eNode, []tsdb.Entity{entity}, nil
+		return ENode, [][]*modelv1.TagValue{entity}, nil
 	case *modelv1.Criteria_Le:
 		le := criteria.GetLe()
 		if le.GetLeft() == nil && le.GetRight() == nil {
 			return nil, nil, errors.WithMessagef(errInvalidLogicalExpression, "both sides(left and right) of [%v] are empty", criteria)
 		}
-		left, leftEntities, err := BuildLocalFilter(le.Left, schema, entityDict, entity)
+		if le.GetLeft() == nil {
+			return BuildLocalFilter(le.Right, schema, entityDict, entity, mandatoryIndexRule)
+		}
+		if le.GetRight() == nil {
+			return BuildLocalFilter(le.Left, schema, entityDict, entity, mandatoryIndexRule)
+		}
+		left, leftEntities, err := BuildLocalFilter(le.Left, schema, entityDict, entity, mandatoryIndexRule)
 		if err != nil {
 			return nil, nil, err
 		}
-		right, rightEntities, err := BuildLocalFilter(le.Right, schema, entityDict, entity)
+		right, rightEntities, err := BuildLocalFilter(le.Right, schema, entityDict, entity, mandatoryIndexRule)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -96,6 +93,9 @@ func BuildLocalFilter(criteria *modelv1.Criteria, schema Schema, entityDict map[
 		if left == nil && right == nil {
 			return nil, entities, nil
 		}
+		if left == ENode && right == ENode {
+			return ENode, entities, nil
+		}
 		switch le.Op {
 		case modelv1.LogicalExpression_LOGICAL_OP_AND:
 			and := newAnd(2)
@@ -110,65 +110,65 @@ func BuildLocalFilter(criteria *modelv1.Criteria, schema Schema, entityDict map[
 	return nil, nil, errInvalidCriteriaType
 }
 
-func parseCondition(cond *modelv1.Condition, indexRule *databasev1.IndexRule, expr LiteralExpr, entity tsdb.Entity) (index.Filter, []tsdb.Entity, error) {
+func parseCondition(cond *modelv1.Condition, indexRule *databasev1.IndexRule, expr LiteralExpr, entity []*modelv1.TagValue) (index.Filter, [][]*modelv1.TagValue, error) {
 	switch cond.Op {
 	case modelv1.Condition_BINARY_OP_GT:
 		return newRange(indexRule, index.RangeOpts{
 			Lower: bytes.Join(expr.Bytes(), nil),
-		}), []tsdb.Entity{entity}, nil
+		}), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_GE:
 		return newRange(indexRule, index.RangeOpts{
 			IncludesLower: true,
 			Lower:         bytes.Join(expr.Bytes(), nil),
-		}), []tsdb.Entity{entity}, nil
+		}), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_LT:
 		return newRange(indexRule, index.RangeOpts{
 			Upper: bytes.Join(expr.Bytes(), nil),
-		}), []tsdb.Entity{entity}, nil
+		}), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_LE:
 		return newRange(indexRule, index.RangeOpts{
 			IncludesUpper: true,
 			Upper:         bytes.Join(expr.Bytes(), nil),
-		}), []tsdb.Entity{entity}, nil
+		}), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_EQ:
-		return newEq(indexRule, expr), []tsdb.Entity{entity}, nil
+		return newEq(indexRule, expr), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_MATCH:
-		return newMatch(indexRule, expr), []tsdb.Entity{entity}, nil
+		return newMatch(indexRule, expr), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_NE:
-		return newNot(indexRule, newEq(indexRule, expr)), []tsdb.Entity{entity}, nil
+		return newNot(indexRule, newEq(indexRule, expr)), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_HAVING:
 		bb := expr.Bytes()
 		and := newAnd(len(bb))
 		for _, b := range bb {
 			and.append(newEq(indexRule, newBytesLiteral(b)))
 		}
-		return and, []tsdb.Entity{entity}, nil
+		return and, [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_NOT_HAVING:
 		bb := expr.Bytes()
 		and := newAnd(len(bb))
 		for _, b := range bb {
 			and.append(newEq(indexRule, newBytesLiteral(b)))
 		}
-		return newNot(indexRule, and), []tsdb.Entity{entity}, nil
+		return newNot(indexRule, and), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_IN:
 		bb := expr.Bytes()
 		or := newOr(len(bb))
 		for _, b := range bb {
 			or.append(newEq(indexRule, newBytesLiteral(b)))
 		}
-		return or, []tsdb.Entity{entity}, nil
+		return or, [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_NOT_IN:
 		bb := expr.Bytes()
 		or := newOr(len(bb))
 		for _, b := range bb {
 			or.append(newEq(indexRule, newBytesLiteral(b)))
 		}
-		return newNot(indexRule, or), []tsdb.Entity{entity}, nil
+		return newNot(indexRule, or), [][]*modelv1.TagValue{entity}, nil
 	}
 	return nil, nil, errors.WithMessagef(errUnsupportedConditionOp, "index filter parses %v", cond)
 }
 
-func parseExprOrEntity(entityDict map[string]int, entity tsdb.Entity, cond *modelv1.Condition) (LiteralExpr, []tsdb.Entity, error) {
+func parseExprOrEntity(entityDict map[string]int, entity []*modelv1.TagValue, cond *modelv1.Condition) (LiteralExpr, [][]*modelv1.TagValue, error) {
 	entityIdx, ok := entityDict[cond.Name]
 	if ok && cond.Op != modelv1.Condition_BINARY_OP_EQ && cond.Op != modelv1.Condition_BINARY_OP_IN {
 		return nil, nil, errors.WithMessagef(errUnsupportedConditionOp, "tag belongs to the entity only supports EQ or IN operation in condition(%v)", cond)
@@ -176,28 +176,25 @@ func parseExprOrEntity(entityDict map[string]int, entity tsdb.Entity, cond *mode
 	switch v := cond.Value.Value.(type) {
 	case *modelv1.TagValue_Str:
 		if ok {
-			parsedEntity := make(tsdb.Entity, len(entity))
+			parsedEntity := make([]*modelv1.TagValue, len(entity))
 			copy(parsedEntity, entity)
-			parsedEntity[entityIdx] = []byte(v.Str.GetValue())
-			return nil, []tsdb.Entity{parsedEntity}, nil
+			parsedEntity[entityIdx] = cond.Value
+			return nil, [][]*modelv1.TagValue{parsedEntity}, nil
 		}
 		return str(v.Str.GetValue()), nil, nil
-	case *modelv1.TagValue_Id:
-		if ok {
-			parsedEntity := make(tsdb.Entity, len(entity))
-			copy(parsedEntity, entity)
-			parsedEntity[entityIdx] = []byte(v.Id.GetValue())
-			return nil, []tsdb.Entity{parsedEntity}, nil
-		}
-		return id(v.Id.GetValue()), nil, nil
-
 	case *modelv1.TagValue_StrArray:
 		if ok && cond.Op == modelv1.Condition_BINARY_OP_IN {
-			entities := make([]tsdb.Entity, len(v.StrArray.Value))
+			entities := make([][]*modelv1.TagValue, len(v.StrArray.Value))
 			for i, va := range v.StrArray.Value {
-				parsedEntity := make(tsdb.Entity, len(entity))
+				parsedEntity := make([]*modelv1.TagValue, len(entity))
 				copy(parsedEntity, entity)
-				parsedEntity[entityIdx] = []byte(va)
+				parsedEntity[entityIdx] = &modelv1.TagValue{
+					Value: &modelv1.TagValue_Str{
+						Str: &modelv1.Str{
+							Value: va,
+						},
+					},
+				}
 				entities[i] = parsedEntity
 			}
 			return nil, entities, nil
@@ -207,21 +204,27 @@ func parseExprOrEntity(entityDict map[string]int, entity tsdb.Entity, cond *mode
 		}, nil, nil
 	case *modelv1.TagValue_Int:
 		if ok {
-			parsedEntity := make(tsdb.Entity, len(entity))
+			parsedEntity := make([]*modelv1.TagValue, len(entity))
 			copy(parsedEntity, entity)
-			parsedEntity[entityIdx] = convert.Int64ToBytes(v.Int.GetValue())
-			return nil, []tsdb.Entity{parsedEntity}, nil
+			parsedEntity[entityIdx] = cond.Value
+			return nil, [][]*modelv1.TagValue{parsedEntity}, nil
 		}
 		return &int64Literal{
 			int64: v.Int.GetValue(),
 		}, nil, nil
 	case *modelv1.TagValue_IntArray:
 		if ok && cond.Op == modelv1.Condition_BINARY_OP_IN {
-			entities := make([]tsdb.Entity, len(v.IntArray.Value))
+			entities := make([][]*modelv1.TagValue, len(v.IntArray.Value))
 			for i, va := range v.IntArray.Value {
-				parsedEntity := make(tsdb.Entity, len(entity))
+				parsedEntity := make([]*modelv1.TagValue, len(entity))
 				copy(parsedEntity, entity)
-				parsedEntity[entityIdx] = convert.Int64ToBytes(va)
+				parsedEntity[entityIdx] = &modelv1.TagValue{
+					Value: &modelv1.TagValue_Int{
+						Int: &modelv1.Int{
+							Value: va,
+						},
+					},
+				}
 				entities[i] = parsedEntity
 			}
 			return nil, entities, nil
@@ -235,13 +238,13 @@ func parseExprOrEntity(entityDict map[string]int, entity tsdb.Entity, cond *mode
 	return nil, nil, errors.WithMessagef(errUnsupportedConditionValue, "index filter parses %v", cond)
 }
 
-func parseEntities(op modelv1.LogicalExpression_LogicalOp, input tsdb.Entity, left, right []tsdb.Entity) []tsdb.Entity {
+func parseEntities(op modelv1.LogicalExpression_LogicalOp, input []*modelv1.TagValue, left, right [][]*modelv1.TagValue) [][]*modelv1.TagValue {
 	count := len(input)
-	result := make(tsdb.Entity, count)
-	anyEntity := func(entities []tsdb.Entity) bool {
+	result := make([]*modelv1.TagValue, count)
+	anyEntity := func(entities [][]*modelv1.TagValue) bool {
 		for _, entity := range entities {
 			for _, entry := range entity {
-				if !bytes.Equal(entry, tsdb.AnyEntry) {
+				if entry != pbv1.AnyTagValue {
 					return false
 				}
 			}
@@ -251,7 +254,7 @@ func parseEntities(op modelv1.LogicalExpression_LogicalOp, input tsdb.Entity, le
 	leftAny := anyEntity(left)
 	rightAny := anyEntity(right)
 
-	mergedEntities := make([]tsdb.Entity, 0, len(left)+len(right))
+	mergedEntities := make([][]*modelv1.TagValue, 0, len(left)+len(right))
 
 	switch op {
 	case modelv1.LogicalExpression_LOGICAL_OP_AND:
@@ -264,15 +267,15 @@ func parseEntities(op modelv1.LogicalExpression_LogicalOp, input tsdb.Entity, le
 		mergedEntities = append(mergedEntities, left...)
 		mergedEntities = append(mergedEntities, right...)
 		for i := 0; i < count; i++ {
-			entry := tsdb.AnyEntry
+			entry := pbv1.AnyTagValue
 			for j := 0; j < len(mergedEntities); j++ {
 				e := mergedEntities[j][i]
-				if bytes.Equal(e, tsdb.AnyEntry) {
+				if e == pbv1.AnyTagValue {
 					continue
 				}
-				if bytes.Equal(entry, tsdb.AnyEntry) {
+				if entry == pbv1.AnyTagValue {
 					entry = e
-				} else if !bytes.Equal(entry, e) {
+				} else if pbv1.MustCompareTagValue(entry, e) != 0 {
 					return nil
 				}
 			}
@@ -288,19 +291,19 @@ func parseEntities(op modelv1.LogicalExpression_LogicalOp, input tsdb.Entity, le
 		mergedEntities = append(mergedEntities, left...)
 		mergedEntities = append(mergedEntities, right...)
 		for i := 0; i < count; i++ {
-			entry := tsdb.AnyEntry
+			entry := pbv1.AnyTagValue
 			for j := 0; j < len(mergedEntities); j++ {
 				e := mergedEntities[j][i]
-				if bytes.Equal(entry, tsdb.AnyEntry) {
+				if entry == pbv1.AnyTagValue {
 					entry = e
-				} else if !bytes.Equal(entry, e) {
+				} else if pbv1.MustCompareTagValue(entry, e) != 0 {
 					return mergedEntities
 				}
 			}
 			result[i] = entry
 		}
 	}
-	return []tsdb.Entity{result}
+	return [][]*modelv1.TagValue{result}
 }
 
 type fieldKey struct {
@@ -632,7 +635,8 @@ func jsonToString(marshaler json.Marshaler) string {
 }
 
 var (
-	eNode = new(emptyNode)
+	// ENode is an empty node.
+	ENode = new(emptyNode)
 	bList = new(bypassList)
 )
 
@@ -648,7 +652,7 @@ func (an emptyNode) String() string {
 
 type bypassList struct{}
 
-func (bl bypassList) Contains(_ common.ItemID) bool {
+func (bl bypassList) Contains(_ uint64) bool {
 	// all items should be fetched
 	return true
 }
@@ -657,7 +661,7 @@ func (bl bypassList) IsEmpty() bool {
 	return false
 }
 
-func (bl bypassList) Max() (common.ItemID, error) {
+func (bl bypassList) Max() (uint64, error) {
 	panic("not invoked")
 }
 
@@ -677,7 +681,7 @@ func (bl bypassList) Equal(_ posting.List) bool {
 	panic("not invoked")
 }
 
-func (bl bypassList) Insert(_ common.ItemID) {
+func (bl bypassList) Insert(_ uint64) {
 	panic("not invoked")
 }
 
@@ -701,11 +705,11 @@ func (bl bypassList) AddIterator(_ posting.Iterator) error {
 	panic("not invoked")
 }
 
-func (bl bypassList) AddRange(_ common.ItemID, _ common.ItemID) error {
+func (bl bypassList) AddRange(_ uint64, _ uint64) error {
 	panic("not invoked")
 }
 
-func (bl bypassList) RemoveRange(_ common.ItemID, _ common.ItemID) error {
+func (bl bypassList) RemoveRange(_ uint64, _ uint64) error {
 	panic("not invoked")
 }
 
@@ -713,7 +717,7 @@ func (bl bypassList) Reset() {
 	panic("not invoked")
 }
 
-func (bl bypassList) ToSlice() []common.ItemID {
+func (bl bypassList) ToSlice() []uint64 {
 	panic("not invoked")
 }
 
