@@ -20,18 +20,16 @@ package measure
 import (
 	"context"
 	"fmt"
-	"io"
 	"time"
 
-	"go.uber.org/multierr"
 	"google.golang.org/protobuf/types/known/timestamppb"
 
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	measurev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/measure/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
-	"github.com/apache/skywalking-banyandb/banyand/tsdb"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
+	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query/executor"
 	"github.com/apache/skywalking-banyandb/pkg/query/logical"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
@@ -50,8 +48,15 @@ type unresolvedIndexScan struct {
 }
 
 func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error) {
+	projTags := make([]pbv1.TagProjection, len(uis.projectionTags))
 	var projTagsRefs [][]*logical.TagRef
 	if len(uis.projectionTags) > 0 {
+		for i := range uis.projectionTags {
+			for _, tag := range uis.projectionTags[i] {
+				projTags[i].Family = tag.GetFamilyName()
+				projTags[i].Names = append(projTags[i].Names, tag.GetTagName())
+			}
+		}
 		var err error
 		projTagsRefs, err = s.CreateTagRef(uis.projectionTags...)
 		if err != nil {
@@ -59,8 +64,12 @@ func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error)
 		}
 	}
 
+	var projField []string
 	var projFieldRefs []*logical.FieldRef
 	if len(uis.projectionFields) > 0 {
+		for i := range uis.projectionFields {
+			projField = append(projField, uis.projectionFields[i].Name)
+		}
 		var err error
 		projFieldRefs, err = s.CreateFieldRef(uis.projectionFields...)
 		if err != nil {
@@ -70,13 +79,13 @@ func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error)
 
 	entityList := s.EntityList()
 	entityMap := make(map[string]int)
-	entity := make([]tsdb.Entry, len(entityList))
+	entity := make([]*modelv1.TagValue, len(entityList))
 	for idx, e := range entityList {
 		entityMap[e] = idx
 		// fill AnyEntry by default
-		entity[idx] = tsdb.AnyEntry
+		entity[idx] = pbv1.AnyTagValue
 	}
-	filter, entities, err := logical.BuildLocalFilter(uis.criteria, s, entityMap, entity)
+	filter, entities, err := logical.BuildLocalFilter(uis.criteria, s, entityMap, entity, true)
 	if err != nil {
 		return nil, err
 	}
@@ -84,12 +93,15 @@ func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error)
 	return &localIndexScan{
 		timeRange:            timestamp.NewInclusiveTimeRange(uis.startTime, uis.endTime),
 		schema:               s,
+		projectionTags:       projTags,
+		projectionFields:     projField,
 		projectionTagsRefs:   projTagsRefs,
 		projectionFieldsRefs: projFieldRefs,
 		metadata:             uis.metadata,
 		filter:               filter,
 		entities:             entities,
 		groupByEntity:        uis.groupByEntity,
+		uis:                  uis,
 		l:                    logger.GetLogger("query", "measure", uis.metadata.Group, uis.metadata.Name, "local-index"),
 	}, nil
 }
@@ -101,17 +113,20 @@ var (
 )
 
 type localIndexScan struct {
-	schema               logical.Schema
 	filter               index.Filter
+	schema               logical.Schema
+	uis                  *unresolvedIndexScan
 	order                *logical.OrderBy
 	metadata             *commonv1.Metadata
 	l                    *logger.Logger
 	timeRange            timestamp.TimeRange
+	projectionTags       []pbv1.TagProjection
 	projectionTagsRefs   [][]*logical.TagRef
 	projectionFieldsRefs []*logical.FieldRef
-	entities             []tsdb.Entity
-	groupByEntity        bool
+	entities             [][]*modelv1.TagValue
+	projectionFields     []string
 	maxDataPointsSize    int
+	groupByEntity        bool
 }
 
 func (i *localIndexScan) Limit(max int) {
@@ -122,68 +137,30 @@ func (i *localIndexScan) Sort(order *logical.OrderBy) {
 	i.order = order
 }
 
-func (i *localIndexScan) Execute(ec executor.MeasureExecutionContext) (executor.MIterator, error) {
-	var seriesList tsdb.SeriesList
-	for _, e := range i.entities {
-		shards, err := ec.Shards(e)
-		if err != nil {
-			return nil, err
-		}
-		for _, shard := range shards {
-			sl, err := shard.Series().List(context.WithValue(
-				context.Background(),
-				logger.ContextKey,
-				i.l,
-			), tsdb.NewPath(e))
-			if err != nil {
-				return nil, err
-			}
-			seriesList = seriesList.Merge(sl)
+func (i *localIndexScan) Execute(ctx context.Context) (mit executor.MIterator, err error) {
+	var orderBy *pbv1.OrderBy
+	if i.order != nil {
+		orderBy = &pbv1.OrderBy{
+			Index: i.order.Index,
+			Sort:  i.order.Sort,
 		}
 	}
-	if len(seriesList) == 0 {
-		return dummyIter, nil
+	ec := executor.FromMeasureExecutionContext(ctx)
+	result, err := ec.Query(ctx, pbv1.MeasureQueryOptions{
+		Name:            i.metadata.GetName(),
+		TimeRange:       &i.timeRange,
+		Entities:        i.entities,
+		Filter:          i.filter,
+		Order:           orderBy,
+		TagProjection:   i.projectionTags,
+		FieldProjection: i.projectionFields,
+	})
+	if err != nil {
+		return nil, fmt.Errorf("failed to query measure: %w", err)
 	}
-	var builders []logical.SeekerBuilder
-	if i.order.Index != nil {
-		builders = append(builders, func(builder tsdb.SeekerBuilder) {
-			builder.OrderByIndex(i.order.Index, i.order.Sort)
-		})
-	} else {
-		builders = append(builders, func(builder tsdb.SeekerBuilder) {
-			builder.OrderByTime(i.order.Sort)
-		})
-	}
-	if i.filter != nil {
-		builders = append(builders, func(b tsdb.SeekerBuilder) {
-			b.Filter(i.filter)
-		})
-	}
-	iters, closers, innerErr := logical.ExecuteForShard(i.l, seriesList, i.timeRange, builders...)
-	if len(closers) > 0 {
-		defer func(closers []io.Closer) {
-			for _, c := range closers {
-				_ = c.Close()
-			}
-		}(closers)
-	}
-	if innerErr != nil {
-		return nil, innerErr
-	}
-
-	if len(iters) == 0 {
-		return dummyIter, nil
-	}
-	transformContext := transformContext{
-		ec:                   ec,
-		projectionTagsRefs:   i.projectionTagsRefs,
-		projectionFieldsRefs: i.projectionFieldsRefs,
-	}
-	if i.groupByEntity {
-		return newSeriesMIterator(iters, transformContext, i.maxDataPointsSize), nil
-	}
-	it := logical.NewItemIter(iters, i.order.Sort)
-	return newIndexScanIterator(it, transformContext, i.maxDataPointsSize), nil
+	return &resultMIterator{
+		result: result,
+	}, nil
 }
 
 func (i *localIndexScan) String() string {
@@ -217,143 +194,58 @@ func indexScan(startTime, endTime time.Time, metadata *commonv1.Metadata, projec
 	}
 }
 
-var _ executor.MIterator = (*indexScanIterator)(nil)
-
-type indexScanIterator struct {
-	inner   logical.ItemIterator
-	err     error
-	current *measurev1.DataPoint
-	context transformContext
-	max     int
-	num     int
-}
-
-func newIndexScanIterator(inner logical.ItemIterator, context transformContext, max int) executor.MIterator {
-	return &indexScanIterator{
-		inner:   inner,
-		context: context,
-		max:     max,
-	}
-}
-
-func (ism *indexScanIterator) Next() bool {
-	if !ism.inner.HasNext() || ism.err != nil || ism.num > ism.max {
-		return false
-	}
-	nextItem := ism.inner.Next()
-	var err error
-	if ism.current, err = transform(nextItem, ism.context); err != nil {
-		ism.err = multierr.Append(ism.err, err)
-	}
-	ism.num++
-	return true
-}
-
-func (ism *indexScanIterator) Current() []*measurev1.DataPoint {
-	if ism.current == nil {
-		return nil
-	}
-	return []*measurev1.DataPoint{ism.current}
-}
-
-func (ism *indexScanIterator) Close() error {
-	return ism.err
-}
-
-var _ executor.MIterator = (*seriesIterator)(nil)
-
-type seriesIterator struct {
-	err     error
-	context transformContext
-	inner   []tsdb.Iterator
+type resultMIterator struct {
+	result  pbv1.MeasureQueryResult
 	current []*measurev1.DataPoint
-	index   int
-	num     int
-	max     int
+	i       int
 }
 
-func newSeriesMIterator(inner []tsdb.Iterator, context transformContext, max int) executor.MIterator {
-	return &seriesIterator{
-		inner:   inner,
-		context: context,
-		index:   -1,
-		max:     max,
+func (ei *resultMIterator) Next() bool {
+	ei.i++
+	if ei.i < len(ei.current) {
+		return true
 	}
-}
 
-func (ism *seriesIterator) Next() bool {
-	if ism.err != nil || ism.num > ism.max {
+	r := ei.result.Pull()
+	if r == nil {
 		return false
 	}
-	ism.index++
-	if ism.index >= len(ism.inner) {
-		return false
-	}
-	iter := ism.inner[ism.index]
-	if ism.current != nil {
-		ism.current = ism.current[:0]
-	}
-	for iter.Next() {
-		dp, err := transform(iter.Val(), ism.context)
-		if err != nil {
-			ism.err = err
-			return false
+	ei.current = ei.current[:0]
+	ei.i = 0
+	for i := range r.Timestamps {
+		dp := &measurev1.DataPoint{
+			Timestamp: timestamppb.New(time.Unix(0, r.Timestamps[i])),
 		}
-		ism.current = append(ism.current, dp)
+
+		for _, tf := range r.TagFamilies {
+			tagFamily := &modelv1.TagFamily{
+				Name: tf.Name,
+			}
+			dp.TagFamilies = append(dp.TagFamilies, tagFamily)
+			for _, t := range tf.Tags {
+				tagFamily.Tags = append(tagFamily.Tags, &modelv1.Tag{
+					Key:   t.Name,
+					Value: t.Values[i],
+				})
+			}
+		}
+		for _, f := range r.Fields {
+			dp.Fields = append(dp.Fields, &measurev1.DataPoint_Field{
+				Name:  f.Name,
+				Value: f.Values[i],
+			})
+		}
+		ei.current = append(ei.current, dp)
 	}
-	ism.num++
+
 	return true
 }
 
-func (ism *seriesIterator) Current() []*measurev1.DataPoint {
-	return ism.current
+func (ei *resultMIterator) Current() []*measurev1.DataPoint {
+	return []*measurev1.DataPoint{ei.current[ei.i]}
 }
 
-func (ism *seriesIterator) Close() error {
-	for _, i := range ism.inner {
-		ism.err = multierr.Append(ism.err, i.Close())
-	}
-	return ism.err
-}
-
-type transformContext struct {
-	ec                   executor.MeasureExecutionContext
-	projectionTagsRefs   [][]*logical.TagRef
-	projectionFieldsRefs []*logical.FieldRef
-}
-
-func transform(item tsdb.Item, ism transformContext) (*measurev1.DataPoint, error) {
-	tagFamilies, err := logical.ProjectItem(ism.ec, item, ism.projectionTagsRefs)
-	if err != nil {
-		return nil, err
-	}
-	dpFields := make([]*measurev1.DataPoint_Field, 0)
-	for _, f := range ism.projectionFieldsRefs {
-		fieldVal, parserFieldErr := ism.ec.ParseField(f.Field.Name, item)
-		if parserFieldErr != nil {
-			return nil, parserFieldErr
-		}
-		dpFields = append(dpFields, fieldVal)
-	}
-	return &measurev1.DataPoint{
-		Fields:      dpFields,
-		TagFamilies: tagFamilies,
-		Timestamp:   timestamppb.New(time.Unix(0, int64(item.Time()))),
-	}, nil
-}
-
-var dummyIter = dummyMIterator{}
-
-type dummyMIterator struct{}
-
-func (ei dummyMIterator) Next() bool {
-	return false
-}
-
-func (ei dummyMIterator) Current() []*measurev1.DataPoint {
-	return nil
-}
-
-func (ei dummyMIterator) Close() error {
+func (ei *resultMIterator) Close() error {
+	ei.result.Release()
 	return nil
 }
