@@ -15,21 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
-package stream
+package stream_test
 
 import (
 	"context"
 	"testing"
 
-	"github.com/golang/mock/gomock"
-	. "github.com/onsi/ginkgo/v2"
-	. "github.com/onsi/gomega"
+	g "github.com/onsi/ginkgo/v2"
+	"github.com/onsi/gomega"
+	"go.uber.org/mock/gomock"
 
-	"github.com/apache/skywalking-banyandb/api/event"
-	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
-	"github.com/apache/skywalking-banyandb/banyand/discovery"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
+	"github.com/apache/skywalking-banyandb/banyand/stream"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/test"
 	"github.com/apache/skywalking-banyandb/pkg/test/flags"
@@ -37,15 +35,15 @@ import (
 )
 
 func TestStream(t *testing.T) {
-	RegisterFailHandler(Fail)
-	RunSpecs(t, "Stream Suite")
+	gomega.RegisterFailHandler(g.Fail)
+	g.RunSpecs(t, "Stream Suite")
 }
 
-var _ = BeforeSuite(func() {
-	Expect(logger.Init(logger.Logging{
+var _ = g.BeforeSuite(func() {
+	gomega.Expect(logger.Init(logger.Logging{
 		Env:   "dev",
 		Level: flags.LogLevel,
-	})).To(Succeed())
+	})).To(gomega.Succeed())
 })
 
 type preloadStreamService struct {
@@ -56,65 +54,50 @@ func (p *preloadStreamService) Name() string {
 	return "preload-stream"
 }
 
-func (p *preloadStreamService) PreRun() error {
-	return teststream.PreloadSchema(p.metaSvc.SchemaRegistry())
+func (p *preloadStreamService) PreRun(ctx context.Context) error {
+	return teststream.PreloadSchema(ctx, p.metaSvc.SchemaRegistry())
 }
 
 type services struct {
-	stream          *service
+	stream          stream.Service
 	metadataService metadata.Service
-	repo            *discovery.MockServiceRepo
 }
 
 func setUp() (*services, func()) {
-	ctrl := gomock.NewController(GinkgoT())
-	Expect(ctrl).ShouldNot(BeNil())
-	// Init Discovery
-	repo := discovery.NewMockServiceRepo(ctrl)
-	repo.EXPECT().NodeID().AnyTimes()
-	repo.EXPECT().Name().AnyTimes()
-	stopCh := make(chan struct{})
-	repo.EXPECT().Serve().Return(stopCh).Times(1)
-	repo.EXPECT().GracefulStop().Do(func() { close(stopCh) }).Times(1)
-	// Both PreRun and Serve phases send events
-	repo.EXPECT().Publish(event.StreamTopicEntityEvent, test.NewEntityEventMatcher(databasev1.Action_ACTION_PUT)).Times(2 * 1)
-	repo.EXPECT().Publish(event.StreamTopicShardEvent, test.NewShardEventMatcher(databasev1.Action_ACTION_PUT)).Times(2 * 2)
-
+	ctrl := gomock.NewController(g.GinkgoT())
+	gomega.Expect(ctrl).ShouldNot(gomega.BeNil())
 	// Init Pipeline
-	pipeline, err := queue.NewQueue(context.TODO(), repo)
-	Expect(err).NotTo(HaveOccurred())
+	pipeline := queue.Local()
 
 	// Init Metadata Service
 	metadataService, err := metadata.NewService(context.TODO())
-	Expect(err).NotTo(HaveOccurred())
-	Expect(err).NotTo(HaveOccurred())
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 	// Init Stream Service
-	streamService, err := NewService(context.TODO(), metadataService, repo, pipeline)
-	Expect(err).NotTo(HaveOccurred())
+	streamService, err := stream.NewService(context.TODO(), metadataService, pipeline)
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	preloadStreamSvc := &preloadStreamService{metaSvc: metadataService}
 	var flags []string
 	metaPath, metaDeferFunc, err := test.NewSpace()
-	Expect(err).NotTo(HaveOccurred())
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	flags = append(flags, "--metadata-root-path="+metaPath)
 	rootPath, deferFunc, err := test.NewSpace()
-	Expect(err).NotTo(HaveOccurred())
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	flags = append(flags, "--stream-root-path="+rootPath)
 	listenClientURL, listenPeerURL, err := test.NewEtcdListenUrls()
-	Expect(err).NotTo(HaveOccurred())
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	flags = append(flags, "--etcd-listen-client-url="+listenClientURL, "--etcd-listen-peer-url="+listenPeerURL)
 	moduleDeferFunc := test.SetupModules(
 		flags,
-		repo,
 		pipeline,
 		metadataService,
 		preloadStreamSvc,
 		streamService,
 	)
 	return &services{
-			stream:          streamService.(*service),
+			stream:          streamService,
 			metadataService: metadataService,
-			repo:            repo,
 		}, func() {
 			moduleDeferFunc()
 			metaDeferFunc()
