@@ -29,9 +29,17 @@ type TimeRange struct {
 	IncludeEnd   bool
 }
 
+// Before returns whether the TimeRange is before the other time.
+func (t TimeRange) Before(other time.Time) bool {
+	if t.IncludeEnd {
+		return t.End.Before(other)
+	}
+	return !t.End.After(other)
+}
+
 // Contains returns whether the unixNano is in the TimeRange.
-func (t TimeRange) Contains(unixNano uint64) bool {
-	tp := time.Unix(0, int64(unixNano))
+func (t TimeRange) Contains(unixNano int64) bool {
+	tp := time.Unix(0, unixNano)
 	if t.Start.Equal(tp) {
 		return t.IncludeStart
 	}
@@ -112,3 +120,31 @@ func NewTimeRange(start, end time.Time, includeStart, includeEnd bool) TimeRange
 func NewTimeRangeDuration(start time.Time, duration time.Duration, includeStart, includeEnd bool) TimeRange {
 	return NewTimeRange(start, start.Add(duration), includeStart, includeEnd)
 }
+
+// FindRange returns the indices of the first and last elements in a sorted 'timestamps' slice that are within the min and max range.
+func FindRange(timestamps []int64, min, max int64) (int, int, bool) {
+	if len(timestamps) == 0 {
+		return -1, -1, false
+	}
+	if timestamps[0] > max {
+		return -1, -1, false
+	}
+	if timestamps[len(timestamps)-1] < min {
+		return -1, -1, false
+	}
+
+	start, end := -1, len(timestamps)
+	for start < len(timestamps)-1 {
+		start++
+		if timestamps[start] >= min {
+			break
+		}
+	}
+	for end > 0 {
+		end--
+		if timestamps[end] <= max {
+			break
+		}
+	}
+	return start, end, start <= end
+}
